from PySide6.QtCore import Qt
from PySide6.QtGui import QColor, QRgba64
from typing import TypeAlias, Union, Any
from enum import Enum


class Colors(Enum):
    _49ers_red: QColor = QColor("#AA0000")
    abalone: QColor = QColor("#D6CFC7")
    abbey: QColor = QColor("#494F55")
    abbey_white: QColor = QColor("#EBE5D0")
    abstract_white: QColor = QColor("#EDE9DD")
    acadia_white: QColor = QColor("#F4EFE3")
    acid_green: QColor = QColor("#B0BF1A")
    acorn: QColor = QColor("#D7A98C")
    acoustic_white: QColor = QColor("#EFECE1")
    admiral: QColor = QColor("#051094")
    admiral_blue: QColor = QColor("#2C3863")
    aegean: QColor = QColor("#1F456E")
    aegean_blue: QColor = QColor("#4E6E81")
    aero_blue: QColor = QColor("#C9FFE5")
    aesthetic_black: QColor = QColor("#1C1C1E")
    aesthetic_orange: QColor = QColor("#F4AB6A")
    aesthetic_purple: QColor = QColor("#502380")
    aesthetic_yellow: QColor = QColor("#EFD033")
    affair: QColor = QColor("#6F4685")
    african_violet: QColor = QColor("#B284BE")
    agreeable_gray: QColor = QColor("#D1CBC1")
    air_force_blue_raf: QColor = QColor("#5D8AA8")
    air_force_blue_usaf: QColor = QColor("#00308F")
    air_superiority_blue: QColor = QColor("#72A0C1")
    airbnb_red: QColor = QColor("#FF5A5F")
    aircraft_white: QColor = QColor("#EDF2F8")
    ajax_red: QColor = QColor("#D2122E")
    alabama_crimson: QColor = QColor("#9E1B32")
    alabaster: QColor = QColor("#EDEAE0")
    alaskan_blue: QColor = QColor("#6DA9D2")
    albescent_white: QColor = QColor("#F3DECA")
    alice_blue: QColor = QColor("#F0F8FF")
    alien_black: QColor = QColor("#1A2228")
    alizarin_crimson: QColor = QColor("#E32636")
    alloy_orange: QColor = QColor("#C35214")
    almond: QColor = QColor("#EFDECD")
    almost_apricot_red: QColor = QColor("#E5B39B")
    amaranth: QColor = QColor("#E52B50")
    amaranth_pink: QColor = QColor("#E52B50")
    amaranth_purple: QColor = QColor("#AB274F")
    amaranth_red: QColor = QColor("#F4364C")
    amazon: QColor = QColor("#3B7A57")
    amber: QColor = QColor("#FFBF00")
    american_rose: QColor = QColor("#FF033E")
    amethyst: QColor = QColor("#9966CC")
    anchor: QColor = QColor("#3E424B")
    android_green: QColor = QColor("#A4C639")
    angels_red: QColor = QColor("#BA0021")
    anti_flash_white: QColor = QColor("#F2F3F4")
    antique_brass: QColor = QColor("#CD9575")
    antique_pewter: QColor = QColor("#808992")
    antique_white: QColor = QColor("#FAEBD7")
    antique_white_usa: QColor = QColor("#EEEAE1")
    anvil_gray: QColor = QColor("#6B6B6B")
    apple: QColor = QColor("#A91B0D")
    apple_green: QColor = QColor("#8DB600")
    apricot: QColor = QColor("#ED820E")
    apricot_orange: QColor = QColor("#FBCEB1")
    aqua_blue: QColor = QColor("#0AFFFF")
    aquamarine_blue: QColor = QColor("#6BCAE2")
    arctic: QColor = QColor("#82EEFD")
    arctic_blue: QColor = QColor("#C6E6FB")
    arctic_lime: QColor = QColor("#D0FF14")
    argentina_blue: QColor = QColor("#6CB4EE")
    army_green: QColor = QColor("#4B5320")
    army_uniform: QColor = QColor("#353F3E")
    arsenal_red: QColor = QColor("#DB0007")
    arsenic: QColor = QColor("#3B444B")
    artichoke_green: QColor = QColor("#8F9779")
    artists_purple: QColor = QColor("#C71585")
    arylide_yellow: QColor = QColor("#E9D66B")
    ash: QColor = QColor("#564C4D")
    ash_brown: QColor = QColor("#98623C")
    ash_gray: QColor = QColor("#B2BEB5")
    ash_white: QColor = QColor("#E9E4D4")
    asparagus: QColor = QColor("#87A96B")
    asphalt: QColor = QColor("#0C0404")
    astros_navy: QColor = QColor("#002D62")
    atomic_tangerine: QColor = QColor("#FF9966")
    atrium_white: QColor = QColor("#F1EEE4")
    auburn: QColor = QColor("#A52A2A")
    auburn_red: QColor = QColor("#A52A2A")
    audience_anger: QColor = QColor("#EAF4FC")
    aureolin: QColor = QColor("#FDEE00")
    autumn_purple: QColor = QColor("#834468")
    avocado: QColor = QColor("#568203")
    aztech_purple: QColor = QColor("#893BFF")
    azure: QColor = QColor("#F0FFFF")
    azureish_white: QColor = QColor("#DBE9F4")
    baby_blue: QColor = QColor("#89CFF0")
    baby_blue_eyes: QColor = QColor("#A1CAF1")
    baby_orange: QColor = QColor("#FCBE85")
    baby_pink: QColor = QColor("#F4C2C2")
    baby_powder: QColor = QColor("#FEFEFA")
    baby_yellow: QColor = QColor("#FCEF91")
    baked_bread: QColor = QColor("#DACBA9")
    baked_potato: QColor = QColor("#B69E87")
    ballet_slipper: QColor = QColor("#F79AC0")
    baltimore_ravens_purple: QColor = QColor("#280353")
    bamboo: QColor = QColor("#E3DEC6")
    banana: QColor = QColor("#FFE135")
    barbie_pink: QColor = QColor("#E0218A")
    bark: QColor = QColor("#996633")
    barn_red: QColor = QColor("#7C0A02")
    basalt_black: QColor = QColor("#4D423E")
    basil: QColor = QColor("#32612D")
    battleship_gray: QColor = QColor("#848482")
    bayern_blue: QColor = QColor("#0066B2")
    bdazzled: QColor = QColor("#2E5894")
    bdazzled_blue: QColor = QColor("#2E5894")
    beau_blue: QColor = QColor("#BCD4E6")
    beaver: QColor = QColor("#9F8170")
    beer_orange: QColor = QColor("#F28F1C")
    beeswax: QColor = QColor("#E9D7AB")
    beige: QColor = QColor("#F5F5DC")
    beige_linen: QColor = QColor("#E2DAC6")
    berry: QColor = QColor("#7A1712")
    berry_blue: QColor = QColor("#4F86F7")
    best_gray: QColor = QColor("#969892")
    bills_red: QColor = QColor("#C60C30")
    birdseed: QColor = QColor("#E2C28E")
    biscotti: QColor = QColor("#E3C565")
    bistre: QColor = QColor("#3D2B1F")
    bitter_chocolate: QColor = QColor("#9E5B40")
    bitter_lemon: QColor = QColor("#CAE00D")
    bitter_lime: QColor = QColor("#BFFF00")
    bittersweet: QColor = QColor("#FE6F5E")
    bittersweet_shimmer: QColor = QColor("#BF4F51")
    bittersweet_shimmer_red: QColor = QColor("#BF4F51")
    black: QColor = QColor("#000000")
    black_bean: QColor = QColor("#3D0C02")
    black_blue_ral: QColor = QColor("#1A1E28")
    black_cat: QColor = QColor("#413839")
    black_chocolate: QColor = QColor("#1B1811")
    black_coral: QColor = QColor("#54626F")
    black_cow: QColor = QColor("#4C4646")
    black_currant: QColor = QColor("#540E32")
    black_denim: QColor = QColor("#191C27")
    black_eel: QColor = QColor("#463E3F")
    black_grain: QColor = QColor("#2C2C2A")
    black_gray: QColor = QColor("#303234")
    black_green_ral: QColor = QColor("#303D3A")
    black_hole: QColor = QColor("#060605")
    black_leather_jacket: QColor = QColor("#253529")
    black_magic: QColor = QColor("#0B0510")
    black_olive: QColor = QColor("#3B3C36")
    black_pearl: QColor = QColor("#0E161A")
    black_plum: QColor = QColor("#362B32")
    black_raspberry: QColor = QColor("#451425")
    black_red: QColor = QColor("#3D2022")
    black_rock: QColor = QColor("#010127")
    black_russian: QColor = QColor("#24252B")
    black_shadows: QColor = QColor("#BFAFB2")
    blackberry: QColor = QColor("#3A3A38")
    blackish_purple: QColor = QColor("#291E29")
    blackstrap_molasses: QColor = QColor("#2B2523")
    blanched_almond: QColor = QColor("#FFEBCD")
    blazing_yellow: QColor = QColor("#FEE715")
    bleu_de_france: QColor = QColor("#318CE7")
    bleu_de_france_blue: QColor = QColor("#318CE7")
    blood: QColor = QColor("#710C04")
    blood_red: QColor = QColor("#660000")
    blue: QColor = QColor("#0000FF")
    blue_charcoal: QColor = QColor("#262B2F")
    blue_gray: QColor = QColor("#98AFC7")
    blue_lilac: QColor = QColor("#7B679A")
    blue_sapphire: QColor = QColor("#126180")
    blue_violet: QColor = QColor("#8A2BE2")
    blue_yonder: QColor = QColor("#5072A7")
    blueberry: QColor = QColor("#241571")
    blush: QColor = QColor("#DE5D83")
    blush_pink: QColor = QColor("#FEC5E5")
    blush_red: QColor = QColor("#BC544B")
    boeing_blue: QColor = QColor("#0039A6")
    bole: QColor = QColor("#79443B")
    bone: QColor = QColor("#E3DAC9")
    bosss_jokes: QColor = QColor("#B0306A")
    bottle_green: QColor = QColor("#006A4E")
    boysenberry: QColor = QColor("#630436")
    brandy: QColor = QColor("#DCB68A")
    brass: QColor = QColor("#B5A642")
    braves_navy: QColor = QColor("#13274F")
    brewers_blue: QColor = QColor("#0A2351")
    brick: QColor = QColor("#7E2811")
    bright_black: QColor = QColor("#222024")
    bright_canary_yellow: QColor = QColor("#FFE41E")
    bright_gray: QColor = QColor("#E8E9EB")
    bright_maroon: QColor = QColor("#C32148")
    bright_mint: QColor = QColor("#4FFFB0")
    bright_pink: QColor = QColor("#FF007F")
    bright_turquoise: QColor = QColor("#08E8DE")
    bright_white: QColor = QColor("#F4F5F0")
    bright_yellow: QColor = QColor("#FFFD01")
    brilliant_purple: QColor = QColor("#D399E6")
    brilliant_rose: QColor = QColor("#FF55A3")
    brilliant_white: QColor = QColor("#EDF1FE")
    brink: QColor = QColor("#FF6090")
    british_racing_green: QColor = QColor("#004225")
    british_standard_bs_4800_10_e_51_yellow: QColor = QColor("#EDC001")
    broken_white: QColor = QColor("#EEEBE3")
    broncos_blue: QColor = QColor("#002244")
    bronze: QColor = QColor("#B2560D")
    bronze_orange: QColor = QColor("#CD7F32")
    bronze_yellow: QColor = QColor("#737000")
    broom_yellow: QColor = QColor("#DBA400")
    brown_bear: QColor = QColor("#7F6244")
    brown_beige: QColor = QColor("#CC8833")
    brown_coffee: QColor = QColor("#4A2C2A")
    brown_gray: QColor = QColor("#8D8468")
    brown_metal: QColor = QColor("#BBADA1")
    brown_oil: QColor = QColor("#231709")
    brown_red: QColor = QColor("#922B05")
    brown_rust: QColor = QColor("#AF593E")
    brown_stone: QColor = QColor("#593C39")
    brown_sugar: QColor = QColor("#A17249")
    brownie: QColor = QColor("#964B00")
    brownish_gray: QColor = QColor("#86775F")
    brownish_red: QColor = QColor("#9E3623")
    browns_orange: QColor = QColor("#FF3C00")
    brunette: QColor = QColor("#3B1E08")
    bu_scarlet: QColor = QColor("#CC0000")
    bubblegum: QColor = QColor("#FFC1CC")
    bubbles: QColor = QColor("#E7FEFF")
    buff_orange: QColor = QColor("#F2CA85")
    buff_yellow: QColor = QColor("#F1BF70")
    bumblebee: QColor = QColor("#FCE205")
    bumblebee_orange: QColor = QColor("#FFC82A")
    burgundy: QColor = QColor("#8D021F")
    burlywood: QColor = QColor("#DEB887")
    burnt_almond: QColor = QColor("#B0724A")
    burnt_henna: QColor = QColor("#7E392F")
    burnt_orange: QColor = QColor("#CC5500")
    burnt_sienna: QColor = QColor("#E97451")
    burnt_umber: QColor = QColor("#8A3324")
    butter: QColor = QColor("#FFFD74")
    butter_toast: QColor = QColor("#D2AD84")
    buttercream: QColor = QColor("#EFE0CD")
    buttermilk: QColor = QColor("#FDEFB2")
    butterscotch_orange: QColor = QColor("#D99151")
    buttery_brown: QColor = QColor("#F1EBDA")
    byzantine_blue: QColor = QColor("#3457D5")
    byzantium: QColor = QColor("#702963")
    cadet_gray: QColor = QColor("#91A3B0")
    cadmium_green: QColor = QColor("#006B3C")
    cadmium_orange: QColor = QColor("#E6812F")
    cadmium_red: QColor = QColor("#E30022")
    cadmium_yellow_lemon: QColor = QColor("#FFF23E")
    cafe_americano: QColor = QColor("#362819")
    cafe_au_lait: QColor = QColor("#A67B5B")
    cafe_noir: QColor = QColor("#4B3621")
    cake_batter: QColor = QColor("#F0EDDB")
    calm_orange: QColor = QColor("#FAB972")
    cambridge_blue: QColor = QColor("#A3C1AD")
    camel: QColor = QColor("#C69F59")
    cameo_pink: QColor = QColor("#EFBBCC")
    camouflage_gray: QColor = QColor("#545851")
    camouflage_green: QColor = QColor("#78866B")
    camouflage_yellow: QColor = QColor("#978942")
    canadian_maple: QColor = QColor("#CAB266")
    canary_yellow: QColor = QColor("#FFEF00")
    candy: QColor = QColor("#D21404")
    candy_apple: QColor = QColor("#FF0800")
    candy_pink: QColor = QColor("#E4717A")
    cantaloupe: QColor = QColor("#FDA172")
    canvas_brown: QColor = QColor("#BB8855")
    capri_blue: QColor = QColor("#00BFFF")
    caramel: QColor = QColor("#AF6F09")
    caramel_apple: QColor = QColor("#B87A59")
    carbon_black: QColor = QColor("#0C0A00")
    carbon_gray: QColor = QColor("#625D5D")
    cardinal: QColor = QColor("#8A244E")
    cardinal_red: QColor = QColor("#C41E3A")
    cardinals_red: QColor = QColor("#BD2031")
    careys: QColor = QColor("#D8A8A8")
    caribbean_green: QColor = QColor("#00CC99")
    carmine: QColor = QColor("#960018")
    carnal_brown: QColor = QColor("#BB8866")
    carnation: QColor = QColor("#FFA6C9")
    carnelian_red: QColor = QColor("#B31B1B")
    carob: QColor = QColor("#362511")
    carolina_blue: QColor = QColor("#4B9CD3")
    carolina_plum: QColor = QColor("#9B84A1")
    carrot: QColor = QColor("#ED7117")
    carrot_orange: QColor = QColor("#ED9121")
    cascading_white: QColor = QColor("#F7F5F6")
    cashmere: QColor = QColor("#D1B399")
    castleton_green: QColor = QColor("#00563B")
    cedar_brown: QColor = QColor("#4A3728")
    celadon: QColor = QColor("#ACE1AF")
    celeste: QColor = QColor("#B2FFFF")
    cement_gray: QColor = QColor("#858A7E")
    ceramic: QColor = QColor("#FCFFF9")
    cerise_pink: QColor = QColor("#EC3B83")
    cerise_red: QColor = QColor("#DE3163")
    cerulean: QColor = QColor("#0492C2")
    cerulean_blue: QColor = QColor("#2A52BE")
    cg_red: QColor = QColor("#E03C31")
    chai: QColor = QColor("#B1832F")
    chameleon_gray: QColor = QColor("#686965")
    chamoisee: QColor = QColor("#A0785A")
    chamomile: QColor = QColor("#DAC395")
    champagne: QColor = QColor("#F7E7CE")
    champagne_pink: QColor = QColor("#F6E1D3")
    charcoal: QColor = QColor("#36454F")
    charcoal_gray: QColor = QColor("#232023")
    charleston_green: QColor = QColor("#232B2B")
    chartreuse: QColor = QColor("#7FFF00")
    chartreuse_traditional: QColor = QColor("#DFFF00")
    chateau: QColor = QColor("#48A860")
    cheese_orange: QColor = QColor("#FFA600")
    chelsea_blue: QColor = QColor("#034694")
    cherry_blossom: QColor = QColor("#FFB7C5")
    cherry_pink: QColor = QColor("#DE3163")
    cherry_red: QColor = QColor("#990F02")
    chestnut: QColor = QColor("#CD5C5C")
    chetwode: QColor = QColor("#F0FFF0")
    chicory: QColor = QColor("#A78658")
    chiefs_red: QColor = QColor("#E31837")
    chiffon: QColor = QColor("#FFFACD")
    chili_red: QColor = QColor("#C21807")
    china_rose: QColor = QColor("#A8516E")
    chinese_green: QColor = QColor("#D0DB61")
    chinese_orange: QColor = QColor("#EB6841")
    chinese_violet: QColor = QColor("#856088")
    chinese_yellow: QColor = QColor("#FFB200")
    chlorine_blue: QColor = QColor("#0CAFFF")
    chlorophyll_green: QColor = QColor("#4AFF00")
    chocolate: QColor = QColor("#7B3F00")
    chocolate_cosmos_red: QColor = QColor("#58111A")
    chocolate_orange: QColor = QColor("#D2691E")
    christmas_green: QColor = QColor("#00873E")
    chrome: QColor = QColor("#DBE2E9")
    cider: QColor = QColor("#B56727")
    cider_orange: QColor = QColor("#D8A175")
    cincinnati_red: QColor = QColor("#C6011F")
    cinereous: QColor = QColor("#98817B")
    cinnabar_red: QColor = QColor("#E44D2E")
    cinnamon_brown: QColor = QColor("#652A0E")
    cinnamon_toast: QColor = QColor("#8D7D77")
    citrine: QColor = QColor("#E4D00A")
    citrus_brown: QColor = QColor("#D7C275")
    clay: QColor = QColor("#80400B")
    cloud: QColor = QColor("#C5C6D0")
    clover_lime: QColor = QColor("#FCE883")
    coal: QColor = QColor("#0C0908")
    cobalt: QColor = QColor("#1338BE")
    cobalt_blue: QColor = QColor("#0047AB")
    coconut: QColor = QColor("#FFF1E6")
    coconut_butter: QColor = QColor("#F2EFE1")
    coconut_milk: QColor = QColor("#FFF1E6")
    coconut_white: QColor = QColor("#E9EDF6")
    coffee: QColor = QColor("#6F4E37")
    coin: QColor = QColor("#9897A9")
    coke_red: QColor = QColor("#F40009")
    cola: QColor = QColor("#F9EDEB")
    college_navy: QColor = QColor("#002244")
    columbia_blue: QColor = QColor("#B9D9EB")
    colza_yellow: QColor = QColor("#EEB700")
    commercial_white: QColor = QColor("#EDECE6")
    cool_black: QColor = QColor("#151922")
    cool_gray: QColor = QColor("#9090C0")
    copper: QColor = QColor("#B87333")
    copper_orange: QColor = QColor("#BB772E")
    copper_red: QColor = QColor("#D97448")
    copper_rose: QColor = QColor("#996666")
    coquelicot_red: QColor = QColor("#FF3800")
    coral: QColor = QColor("#FF7F50")
    coral_pink: QColor = QColor("#FE7D6A")
    coral_red: QColor = QColor("#FF7F50")
    cordovan_red: QColor = QColor("#893F45")
    corn: QColor = QColor("#FBEC5D")
    corn_green: QColor = QColor("#71A92C")
    cornflower_blue: QColor = QColor("#6495ED")
    cornsilk: QColor = QColor("#FFF8DC")
    cosmic_latte: QColor = QColor("#FFF8E7")
    cotton: QColor = QColor("#FBFBF9")
    cotton_ball: QColor = QColor("#F2F7FD")
    cotton_candy: QColor = QColor("#FFBCD9")
    cough_mixture: QColor = QColor("#F6F4F1")
    crayola_blue: QColor = QColor("#1F75FE")
    crayola_orange: QColor = QColor("#FF7538")
    crayola_red: QColor = QColor("#EE204D")
    crayola_yellow_green: QColor = QColor("#ACDF87")
    cream: QColor = QColor("#FFFDD0")
    crimson: QColor = QColor("#B80F0A")
    crocodile: QColor = QColor("#607D3B")
    croissant: QColor = QColor("#C4AB86")
    crow: QColor = QColor("#0D0907")
    cup_cake: QColor = QColor("#EBF6F7")
    currant: QColor = QColor("#680C07")
    cyan_blue: QColor = QColor("#00FFFF")
    cyber_grape_purple: QColor = QColor("#58427C")
    cyber_yellow: QColor = QColor("#FFD300")
    cynical_black: QColor = QColor("#171717")
    daffodil_yellow: QColor = QColor("#E28D00")
    dahlia_yellow_ral: QColor = QColor("#F39B1B")
    daisy: QColor = QColor("#FAFAFA")
    daisy_yellow: QColor = QColor("#F8DC75")
    dallas_cowboys_blue: QColor = QColor("#041E42")
    dandelion: QColor = QColor("#F0E130")
    dark_beige: QColor = QColor("#AC9362")
    dark_black: QColor = QColor("#010203")
    dark_blue: QColor = QColor("#00008B")
    dark_brown: QColor = QColor("#654321")
    dark_byzantium: QColor = QColor("#5D3954")
    dark_charcoal: QColor = QColor("#333333")
    dark_chestnut: QColor = QColor("#986960")
    dark_chocolate: QColor = QColor("#2E1503")
    dark_coral: QColor = QColor("#CD5B45")
    dark_gray: QColor = QColor("#A9A9A9")
    dark_jungle_green: QColor = QColor("#1A2421")
    dark_khaki: QColor = QColor("#9B8F55")
    dark_lemon_lime: QColor = QColor("#76BA1B")
    dark_magenta: QColor = QColor("#8B008B")
    dark_moss_green: QColor = QColor("#4A5D23")
    dark_mulberry: QColor = QColor("#4C0121")
    dark_olive_green: QColor = QColor("#556B2F")
    dark_orange: QColor = QColor("#FF8C00")
    dark_orchid: QColor = QColor("#9932CC")
    dark_pastel_red: QColor = QColor("#C23B22")
    dark_pink: QColor = QColor("#E75480")
    dark_purple: QColor = QColor("#301934")
    dark_raisin: QColor = QColor("#1A0F0F")
    dark_red: QColor = QColor("#8B0000")
    dark_salmon: QColor = QColor("#E9967A")
    dark_sand: QColor = QColor("#9F7D4F")
    dark_sapphire: QColor = QColor("#082567")
    dark_sea_green: QColor = QColor("#8FBC8F")
    dark_sienna: QColor = QColor("#3C1414")
    dark_silver: QColor = QColor("#746D69")
    dark_slate: QColor = QColor("#3E3D53")
    dark_slate_gray: QColor = QColor("#2F4F4F")
    dark_spring_green: QColor = QColor("#177245")
    dark_terra_cotta: QColor = QColor("#CC4E5C")
    dark_topaz_orange: QColor = QColor("#EABD8C")
    dark_turquoise: QColor = QColor("#00CED1")
    dark_vanilla: QColor = QColor("#D1BEA8")
    dark_violet: QColor = QColor("#9400D3")
    dark_wood: QColor = QColor("#855E42")
    dark_yellow: QColor = QColor("#D5B60A")
    dartmouth_green: QColor = QColor("#00703C")
    davys_gray: QColor = QColor("#555555")
    debian_red: QColor = QColor("#D70A53")
    decorators_white: QColor = QColor("#ECEFEC")
    deep_black: QColor = QColor("#050203")
    deep_brown: QColor = QColor("#410200")
    deep_chrome_yellow_ferrario: QColor = QColor("#FFB045")
    deep_coffee: QColor = QColor("#704241")
    deep_green: QColor = QColor("#056608")
    deep_jungle_green: QColor = QColor("#004B49")
    deep_lilac: QColor = QColor("#B65FCF")
    deep_magenta: QColor = QColor("#CC00CC")
    deep_napoli_yellow: QColor = QColor("#FFCF99")
    deep_pink: QColor = QColor("#FF1493")
    deep_pruce: QColor = QColor("#A95C68")
    deep_purple: QColor = QColor("#524F81")
    deep_ruby_purple: QColor = QColor("#843F5B")
    deep_saffron: QColor = QColor("#FFA52C")
    deep_space_sparkle: QColor = QColor("#4A646C")
    deep_taupe: QColor = QColor("#7B6660")
    deep_wine: QColor = QColor("#2C041C")
    deep_yellow: QColor = QColor("#EACE09")
    deer: QColor = QColor("#BA8759")
    delft_blue: QColor = QColor("#1F305E")
    delicate_white: QColor = QColor("#F1F2EE")
    dell_blue: QColor = QColor("#0076CE")
    denim: QColor = QColor("#151E3D")
    denim_blue: QColor = QColor("#1560BD")
    desert_gray: QColor = QColor("#B8A487")
    desert_orange: QColor = QColor("#ED8E4A")
    desert_sand_orange: QColor = QColor("#EDC9AF")
    desire: QColor = QColor("#EA3C53")
    dijon: QColor = QColor("#C49102")
    dim_gray: QColor = QColor("#696969")
    dirt: QColor = QColor("#9B7653")
    dodgers_blue: QColor = QColor("#005A9C")
    dollar_bill: QColor = QColor("#85BB65")
    dolphins_aqua: QColor = QColor("#008E97")
    dorian_gray: QColor = QColor("#ACA79E")
    dove: QColor = QColor("#7C6E7F")
    dragon_fire_orange: QColor = QColor("#FD652D")
    dried_moss: QColor = QColor("#CCB97E")
    duck_blue: QColor = QColor("#007791")
    duke_blue: QColor = QColor("#012169")
    dull_black: QColor = QColor("#161616")
    dull_brown: QColor = QColor("#876E4B")
    dull_gray: QColor = QColor("#8B837E")
    dull_orange: QColor = QColor("#DB804E")
    dull_purple: QColor = QColor("#9861A5")
    dull_yellow: QColor = QColor("#DECA57")
    durian_yellow: QColor = QColor("#E1BD27")
    dust_storm: QColor = QColor("#E5CCC9")
    dusty_gray: QColor = QColor("#B29CA2")
    dusty_orange: QColor = QColor("#E27A53")
    dusty_yellow: QColor = QColor("#D4CC9A")
    dutch_orange: QColor = QColor("#FF9B00")
    dutch_white: QColor = QColor("#EFDFBB")
    early_bird: QColor = QColor("#CEA2FD")
    earth_yellow: QColor = QColor("#E1A95F")
    east_carolina_university_purple: QColor = QColor("#592A8A")
    ebony: QColor = QColor("#555D50")
    echo: QColor = QColor("#898D86")
    ecru: QColor = QColor("#C2B280")
    eerie_black: QColor = QColor("#1B1B1B")
    egg_nog: QColor = QColor("#FAE29C")
    eggplant: QColor = QColor("#311432")
    eggshell: QColor = QColor("#F0EAD6")
    egyptian_blue: QColor = QColor("#1034A6")
    electric_black: QColor = QColor("#292929")
    electric_blue: QColor = QColor("#7DF9FF")
    electric_crimson: QColor = QColor("#FF003F")
    electric_gray: QColor = QColor("#A5ADAD")
    electric_indigo: QColor = QColor("#6F00FF")
    electric_lavender: QColor = QColor("#E39FF6")
    electric_purple: QColor = QColor("#BF00FF")
    electric_violet: QColor = QColor("#8F00FF")
    electric_yellow: QColor = QColor("#FFFF33")
    embassy_purple: QColor = QColor("#3B343C")
    emerald: QColor = QColor("#028A0F")
    emerald_green: QColor = QColor("#50C878")
    eminence: QColor = QColor("#6C3082")
    empire_yellow: QColor = QColor("#F7D000")
    english_red: QColor = QColor("#AB4B52")
    english_violet: QColor = QColor("#563C5C")
    everton_blue: QColor = QColor("#003399")
    facebook_blue: QColor = QColor("#1877F2")
    faded_orange: QColor = QColor("#CD946A")
    faded_purple: QColor = QColor("#795F80")
    faded_yellow: QColor = QColor("#ECE3A1")
    fairy_tale: QColor = QColor("#F2C1D1")
    falcons_red: QColor = QColor("#A71930")
    fall_harvest: QColor = QColor("#A78A59")
    falu_red: QColor = QColor("#801818")
    fandango: QColor = QColor("#B53389")
    fashion_black: QColor = QColor("#060403")
    fashion_gray: QColor = QColor("#8E8D8C")
    fawn: QColor = QColor("#C8A951")
    feather_brown: QColor = QColor("#B8AD9E")
    feather_white: QColor = QColor("#E7EAE5")
    fed_ex_purple: QColor = QColor("#660099")
    feldgrau: QColor = QColor("#4D5D53")
    fern: QColor = QColor("#5DBB63")
    fern_green: QColor = QColor("#4F7942")
    ferrari: QColor = QColor("#FF2800")
    fire: QColor = QColor("#DD571C")
    fire_brick: QColor = QColor("#B22222")
    fire_engine_red: QColor = QColor("#CE2029")
    fire_yellow: QColor = QColor("#FEDE17")
    firewood: QColor = QColor("#CEBAA8")
    flame: QColor = QColor("#E34A27")
    flame_red: QColor = QColor("#E25822")
    flamingo: QColor = QColor("#FC8EAC")
    flax: QColor = QColor("#EEDC82")
    flint: QColor = QColor("#7E7D9C")
    floral_white: QColor = QColor("#FFFAF0")
    fluorescent_blue: QColor = QColor("#15F4EE")
    fluorescent_yellow: QColor = QColor("#CCFF00")
    fog: QColor = QColor("#655967")
    folly_red: QColor = QColor("#FF004F")
    ford_blue: QColor = QColor("#2C3968")
    forest_green: QColor = QColor("#0B6623")
    fortune_cookie: QColor = QColor("#E0C5A1")
    fossil: QColor = QColor("#787276")
    french_beige: QColor = QColor("#A67B50")
    french_fuchsia: QColor = QColor("#FD3F92")
    french_gray: QColor = QColor("#B4B19A")
    french_lime: QColor = QColor("#9EFD38")
    french_mauve: QColor = QColor("#D473D4")
    french_pink: QColor = QColor("#F64A8A")
    french_puce_red: QColor = QColor("#4E1609")
    frost: QColor = QColor("#FCFBFC")
    frost_black: QColor = QColor("#191C20")
    fuchsia: QColor = QColor("#FF00FF")
    fuchsia_blue: QColor = QColor("#9C51B6")
    fulvous: QColor = QColor("#E68000")
    fun: QColor = QColor("#007848")
    fuzzy_wuzzy_red: QColor = QColor("#CC6666")
    gainsboro: QColor = QColor("#DCDCDC")
    gamboge: QColor = QColor("#E89611")
    garnet: QColor = QColor("#610C04")
    garnet_red: QColor = QColor("#733635")
    general_motors_blue: QColor = QColor("#073980")
    gentle_gray: QColor = QColor("#DADADA")
    ghost_white: QColor = QColor("#F8F8FF")
    giants_orange: QColor = QColor("#FE5A1D")
    gin: QColor = QColor("#D8E4BC")
    ginger: QColor = QColor("#B06500")
    ginger_ale: QColor = QColor("#C9A86A")
    ginger_orange: QColor = QColor("#B06500")
    gingerbread: QColor = QColor("#5E2C04")
    girl_scout_green: QColor = QColor("#00AE58")
    glaucous: QColor = QColor("#6082B6")
    glaucous_blue: QColor = QColor("#6082B6")
    glitter: QColor = QColor("#E6E8FA")
    glossy_black: QColor = QColor("#252324")
    glossy_gold: QColor = QColor("#FAE481")
    glossy_gray: QColor = QColor("#AFADA9")
    glossy_orange: QColor = QColor("#E78400")
    go_green: QColor = QColor("#00AB66")
    gold: QColor = QColor("#FFD700")
    gold_pink: QColor = QColor("#E6C7C2")
    golden_brown: QColor = QColor("#996515")
    golden_fog: QColor = QColor("#F0EAD2")
    golden_gate_bridge_orange: QColor = QColor("#C0362C")
    golden_poppy: QColor = QColor("#FCC200")
    golden_yellow_ral: QColor = QColor("#DD9F00")
    goldenrod: QColor = QColor("#DAA520")
    goldenrod_orange: QColor = QColor("#DAA520")
    goldfish_orange: QColor = QColor("#FF9913")
    good_tax: QColor = QColor("#C9A0FF")
    gothic_grape: QColor = QColor("#120321")
    granite: QColor = QColor("#676767")
    granny_smith_apple: QColor = QColor("#A8E4A0")
    granola: QColor = QColor("#D6B85A")
    grape: QColor = QColor("#663046")
    grape_color: QColor = QColor("#6F2DA8")
    grapefruit_pink: QColor = QColor("#E0707C")
    graphite: QColor = QColor("#594D5B")
    graphite_black_ral: QColor = QColor("#27292B")
    graphite_gray: QColor = QColor("#4B4E53")
    gray: QColor = QColor("#808080")
    gray_brown: QColor = QColor("#7F7053")
    gray_cloud: QColor = QColor("#B6B6B4")
    gray_cmyk: QColor = QColor("#939598")
    gray_dolphin: QColor = QColor("#5C5858")
    gray_goose: QColor = QColor("#D1D0CE")
    gray_green: QColor = QColor("#5E716A")
    gray_owl: QColor = QColor("#D5D5CD")
    gray_wolf: QColor = QColor("#504A4B")
    grayish_brown: QColor = QColor("#7A6A4F")
    grease: QColor = QColor("#080806")
    greek_orange: QColor = QColor("#E06E04")
    greek_villa: QColor = QColor("#F0EBE3")
    green: QColor = QColor("#3CB043")
    green_lizard: QColor = QColor("#A7F432")
    green_munsell: QColor = QColor("#00A877")
    green_screen: QColor = QColor("#00B140")
    green_yellow: QColor = QColor("#ADFF2F")
    greenish_yellow: QColor = QColor("#EEEA62")
    gun_smoke_gray: QColor = QColor("#8A868E")
    gunmetal: QColor = QColor("#2A3439")
    guppie_green: QColor = QColor("#00FF7F")
    half_and_half: QColor = QColor("#FFFFE4")
    halloween_orange: QColor = QColor("#EE5921")
    harbor_gray: QColor = QColor("#C7C6C1")
    hardwood: QColor = QColor("#937A62")
    harlequin: QColor = QColor("#3FFF00")
    harp: QColor = QColor("#EBF5F0")
    harvard_crimson: QColor = QColor("#A51C30")
    hawaiian_brown: QColor = QColor("#99522C")
    hazel_wood: QColor = QColor("#C9BB8E")
    hazelnut: QColor = QColor("#BDA55D")
    heather: QColor = QColor("#9AA297")
    heavy_brown: QColor = QColor("#73624A")
    heavy_gray: QColor = QColor("#050505")
    heliotrope: QColor = QColor("#DF73FF")
    hibiscus: QColor = QColor("#B43757")
    hickory: QColor = QColor("#371D10")
    hippie: QColor = QColor("#A84860")
    hit: QColor = QColor("#FFA878")
    hog_bristle_quarter: QColor = QColor("#EBE4D8")
    hollywood_cerise: QColor = QColor("#F400A1")
    hollywood_cerise_red: QColor = QColor("#F400A1")
    honey: QColor = QColor("#FFC30B")
    honey_brown: QColor = QColor("#BA9238")
    honey_orange: QColor = QColor("#FFBD31")
    honeydew: QColor = QColor("#F0FFF0")
    hookers_green: QColor = QColor("#49796B")
    hopbush: QColor = QColor("#D05FAD")
    hot_orange: QColor = QColor("#FF6E00")
    hot_pink: QColor = QColor("#FF69B4")
    hot_purple: QColor = QColor("#A420D0")
    hot_yellow: QColor = QColor("#FFDA00")
    hunter_green: QColor = QColor("#355E3B")
    huntington_white: QColor = QColor("#F1F0E5")
    hunyadi_orange: QColor = QColor("#E8AC41")
    hunyadi_yellow: QColor = QColor("#E8AC41")
    husker_red: QColor = QColor("#E41C38")
    ibm_blue: QColor = QColor("#0530AD")
    ice_blue: QColor = QColor("#99FFFF")
    ice_cube: QColor = QColor("#E3E4E0")
    ice_gray: QColor = QColor("#BEC3C6")
    iced_coffee: QColor = QColor("#B18F6A")
    icterine: QColor = QColor("#FCF75E")
    iguana_green: QColor = QColor("#71BC78")
    imperial_blue: QColor = QColor("#005A92")
    imperial_purple: QColor = QColor("#602F6B")
    imperial_red: QColor = QColor("#ED2939")
    inchworm: QColor = QColor("#B2EC5D")
    independence_blue: QColor = QColor("#4C516D")
    india_green: QColor = QColor("#138808")
    india_saffron: QColor = QColor("#FF9933")
    indian_red: QColor = QColor("#CD5C5C")
    indian_yellow: QColor = QColor("#E3A857")
    indigo: QColor = QColor("#4B0082")
    indigo_blue: QColor = QColor("#3F00FF")
    indigo_dye_blue: QColor = QColor("#00416A")
    ink: QColor = QColor("#070504")
    ink_black_ral: QColor = QColor("#212122")
    insolent_purple: QColor = QColor("#682E3C")
    intel_blue: QColor = QColor("#0071C5")
    international_klein_blue: QColor = QColor("#002FA7")
    international_orange: QColor = QColor("#FF4F00")
    iridium: QColor = QColor("#3D3C3A")
    iris: QColor = QColor("#9867C5")
    iris_blue: QColor = QColor("#5A4FCF")
    irish_green: QColor = QColor("#009A44")
    irish_orange: QColor = QColor("#F5883F")
    iron: QColor = QColor("#A59C94")
    iron_black: QColor = QColor("#343432")
    isabelline: QColor = QColor("#F4F0EC")
    island_spice: QColor = QColor("#FFFCEC")
    ivory: QColor = QColor("#FFFFF0")
    ivory_black: QColor = QColor("#231F20")
    jade: QColor = QColor("#000302")
    jade_green: QColor = QColor("#00A86B")
    jam: QColor = QColor("#67032F")
    japanese_carmine: QColor = QColor("#9D2933")
    japanese_violet_purple: QColor = QColor("#5B3256")
    jasper_orange: QColor = QColor("#E89149")
    jasper_red: QColor = QColor("#D73B3E")
    jazzberry_jam: QColor = QColor("#67032F")
    jelly_bean_red: QColor = QColor("#DA614E")
    jet: QColor = QColor("#343434")
    jet_black: QColor = QColor("#030303")
    jet_gray: QColor = QColor("#616D7E")
    john_deere_green: QColor = QColor("#367C2B")
    jungle_green: QColor = QColor("#29AB87")
    juniper: QColor = QColor("#3A5311")
    kaitoke: QColor = QColor("#004830")
    kalamata: QColor = QColor("#705160")
    kc_royals_blue: QColor = QColor("#004687")
    kelly_green: QColor = QColor("#4CBB17")
    kenyan_copper_red: QColor = QColor("#7C1C05")
    khaki: QColor = QColor("#C3B091")
    kikyo_iru: QColor = QColor("#5D3F6A")
    kingfisher_daisy: QColor = QColor("#653780")
    knockout_pink: QColor = QColor("#FF3EA5")
    kobicha: QColor = QColor("#6B4423")
    koi_orange: QColor = QColor("#D15837")
    kokushoku: QColor = QColor("#171412")
    kombu_green: QColor = QColor("#354230")
    ksu_purple: QColor = QColor("#512888")
    kurotobi: QColor = QColor("#351E1C")
    la_salle_green: QColor = QColor("#087830")
    lace: QColor = QColor("#F8F2ED")
    lady: QColor = QColor("#F0C0A8")
    laguna: QColor = QColor("#F8E473")
    laid_back_gray: QColor = QColor("#B4AFA9")
    lakers_purple: QColor = QColor("#552582")
    lamp_black: QColor = QColor("#2E473B")
    languid_lavender: QColor = QColor("#D6CADD")
    lapis: QColor = QColor("#2832C2")
    lapis_blue: QColor = QColor("#26619C")
    laser_lemon: QColor = QColor("#FEFE22")
    latte: QColor = QColor("#C5A582")
    laurel_green: QColor = QColor("#A9BA9D")
    lava: QColor = QColor("#808588")
    lava_orange: QColor = QColor("#F76806")
    lava_red: QColor = QColor("#CF1020")
    lavender_blush: QColor = QColor("#FFF0F5")
    lavender_gray: QColor = QColor("#C4C3D0")
    lavender_rose: QColor = QColor("#FBA0E3")
    lawn_green: QColor = QColor("#7CFC00")
    lead: QColor = QColor("#3F3F4E")
    leather: QColor = QColor("#0B0705")
    lemon_chiffon: QColor = QColor("#FFFACD")
    lemon_curry: QColor = QColor("#CCA01D")
    lemon_yellow: QColor = QColor("#FDFF00")
    lemonade: QColor = QColor("#F2DBE7")
    lemonade_crepe: QColor = QColor("#F2B8C6")
    lexicon: QColor = QColor("#E7EAEA")
    lexicon_quarter: QColor = QColor("#F2F2F1")
    liberty_blue: QColor = QColor("#545AA7")
    licorice: QColor = QColor("#1A1110")
    light_blush: QColor = QColor("#F1ABB9")
    light_brick: QColor = QColor("#FB607F")
    light_brown: QColor = QColor("#B5651D")
    light_chrome_yellow: QColor = QColor("#FFCA37")
    light_copper_orange: QColor = QColor("#F6CBA0")
    light_coral: QColor = QColor("#F08080")
    light_crimson: QColor = QColor("#F56991")
    light_deep_pink: QColor = QColor("#FF5CCD")
    light_goldenrod: QColor = QColor("#FAFAD2")
    light_gray: QColor = QColor("#D3D3D3")
    light_gunmetal: QColor = QColor("#344148")
    light_hot_pink: QColor = QColor("#FFB3DE")
    light_khaki: QColor = QColor("#F0E68C")
    light_orange: QColor = QColor("#FED8B1")
    light_pink: QColor = QColor("#FFB6C1")
    light_red_ochre: QColor = QColor("#E97451")
    light_saffron: QColor = QColor("#FFE797")
    light_salmon: QColor = QColor("#FDAB9F")
    light_slate_gray: QColor = QColor("#6D7B8D")
    light_taupe: QColor = QColor("#B38B6D")
    light_turquoise: QColor = QColor("#AFEEEE")
    light_yellow: QColor = QColor("#FFFFE0")
    lightsaber_green: QColor = QColor("#2FF924")
    lilac: QColor = QColor("#B65FCF")
    lime: QColor = QColor("#AEF359")
    lime_green: QColor = QColor("#C7EA46")
    limerick: QColor = QColor("#9DC209")
    linen: QColor = QColor("#F3EAD3")
    link_white: QColor = QColor("#ECF3F9")
    lip_pink: QColor = QColor("#DBAC98")
    lipstick: QColor = QColor("#9B1003")
    lipstick_stain: QColor = QColor("#8E4785")
    liserian_purple: QColor = QColor("#DE6FA1")
    little_boy_blue: QColor = QColor("#6CA0DC")
    little_princess: QColor = QColor("#E9DCE5")
    liver: QColor = QColor("#534B4F")
    liverpool_red: QColor = QColor("#C8102E")
    long_distance: QColor = QColor("#6F456E")
    lovely_purple: QColor = QColor("#7F38EC")
    lowes_blue: QColor = QColor("#004792")
    luminous_bright_orange_ral: QColor = QColor("#FFB700")
    lust_red: QColor = QColor("#E62020")
    luxury_black: QColor = QColor("#060D0D")
    luxury_purple: QColor = QColor("#743089")
    macaroon: QColor = QColor("#F9E076")
    madder_red: QColor = QColor("#A50021")
    magenta: QColor = QColor("#FF00FF")
    magenta_process: QColor = QColor("#E11584")
    magenta_purple: QColor = QColor("#A32CC4")
    magic_blue: QColor = QColor("#0077C0")
    magnolia: QColor = QColor("#F8F4FF")
    mahogany: QColor = QColor("#C04000")
    mahogany_orange: QColor = QColor("#C04000")
    maize_yellow_ral: QColor = QColor("#DB9100")
    majorelle_blue: QColor = QColor("#6050DC")
    malachite: QColor = QColor("#0BDA51")
    manchester_united_red: QColor = QColor("#DA291C")
    mango_tango: QColor = QColor("#FB8842")
    mantis: QColor = QColor("#74C365")
    maple_sugar: QColor = QColor("#C9A38D")
    maple_syrup: QColor = QColor("#BB9351")
    mardi_gras: QColor = QColor("#880085")
    marengo: QColor = QColor("#4C5866")
    marian_blue: QColor = QColor("#E1EBEE")
    marigold: QColor = QColor("#FCB810")
    marigold_orange: QColor = QColor("#EAA221")
    marmalade: QColor = QColor("#D16002")
    maroon: QColor = QColor("#800000")
    mars_brown: QColor = QColor("#AD6242")
    matte_purple: QColor = QColor("#392A48")
    matte_silver: QColor = QColor("#A5A5A8")
    matte_yellow: QColor = QColor("#EDD94C")
    mauve: QColor = QColor("#E0B0FF")
    mauveine: QColor = QColor("#8D029B")
    mauvelous: QColor = QColor("#EF98AA")
    mavericks_blue: QColor = QColor("#00538C")
    maximum_green: QColor = QColor("#4C9A2A")
    maximum_green_yellow: QColor = QColor("#D9E650")
    maximum_yellow: QColor = QColor("#FAFA37")
    maya_blue: QColor = QColor("#73C2FB")
    mecca_orange: QColor = QColor("#BD5745")
    medium_blue: QColor = QColor("#0000CD")
    medium_brown: QColor = QColor("#7F5112")
    medium_gray: QColor = QColor("#BEBEBE")
    medium_jungle_green: QColor = QColor("#1C352D")
    medium_orchid: QColor = QColor("#BA55D3")
    medium_purple: QColor = QColor("#9370DB")
    medium_purple_3: QColor = QColor("#8968CD")
    medium_sapphire: QColor = QColor("#2D5DA1")
    medium_sea_green: QColor = QColor("#3CB371")
    medium_slate_blue: QColor = QColor("#7B68EE")
    medium_spring_bud: QColor = QColor("#C9DC87")
    medium_spring_green: QColor = QColor("#00FA9A")
    medium_turquoise: QColor = QColor("#48D1CC")
    medium_vermilion: QColor = QColor("#CF5B2E")
    medium_vermillion: QColor = QColor("#D9603B")
    medium_violet_red: QColor = QColor("#C71585")
    mellow_yellow: QColor = QColor("#F8DE7E")
    melon: QColor = QColor("#F7BCAC")
    melon_orange: QColor = QColor("#DF9166")
    melon_yellow: QColor = QColor("#FF9C00")
    mercury: QColor = QColor("#E7E7E7")
    merigold: QColor = QColor("#FCAE1E")
    merino: QColor = QColor("#F2EBDD")
    merlot: QColor = QColor("#541E1B")
    metal: QColor = QColor("#0E0C0A")
    metallic_gray: QColor = QColor("#A8A9AD")
    metallic_orange: QColor = QColor("#E26310")
    metallic_purple: QColor = QColor("#520E7D")
    metropolis: QColor = QColor("#1A1A1A")
    mexican_pink: QColor = QColor("#E4007C")
    middle_green: QColor = QColor("#4D8C57")
    middle_purple: QColor = QColor("#D982B5")
    midnight: QColor = QColor("#2B1B17")
    midnight_green: QColor = QColor("#004953")
    mikado_yellow: QColor = QColor("#FFC40C")
    milano: QColor = QColor("#D95D67")
    milk: QColor = QColor("#FDFFF5")
    millennium_blue: QColor = QColor("#002244")
    mimosa: QColor = QColor("#FFF4BC")
    mimosa_orange: QColor = QColor("#FFCA4B")
    mineral_yellow: QColor = QColor("#D39C43")
    minion_yellow: QColor = QColor("#F5E050")
    mink: QColor = QColor("#888078")
    mint: QColor = QColor("#99EDC3")
    mint_cream: QColor = QColor("#F5FFFA")
    mint_green: QColor = QColor("#98FB98")
    misty_gray: QColor = QColor("#BCC2C2")
    misty_rose: QColor = QColor("#FFE4E1")
    mit_red: QColor = QColor("#A31F34")
    moccasin: QColor = QColor("#FFE4B5")
    mocha: QColor = QColor("#3C280D")
    mojave_desert: QColor = QColor("#C7B595")
    morning_blue: QColor = QColor("#8DA399")
    moss: QColor = QColor("#466D1D")
    moss_green: QColor = QColor("#8A9A5B")
    mountains_majesty: QColor = QColor("#9078C0")
    mountbatten_pink: QColor = QColor("#997A8D")
    mughal_green: QColor = QColor("#306030")
    mulberry: QColor = QColor("#C54B8C")
    munsell_blue: QColor = QColor("#0093AF")
    munsell_purple: QColor = QColor("#9F00C5")
    munsell_red: QColor = QColor("#F2003C")
    munsell_yellow: QColor = QColor("#EFCC00")
    murasaki_purple: QColor = QColor("#4F284B")
    mustard: QColor = QColor("#FEDC56")
    muted_yellow: QColor = QColor("#E0D268")
    myrtle_green: QColor = QColor("#317873")
    mythical_purple: QColor = QColor("#53277E")
    nadeshiko_pink: QColor = QColor("#F6ADC6")
    napier_green: QColor = QColor("#2A8000")
    nationals_red: QColor = QColor("#AB0003")
    natural_black: QColor = QColor("#07000B")
    natural_white: QColor = QColor("#EEECE6")
    navajo_white: QColor = QColor("#FFDEAD")
    navy: QColor = QColor("#0A1172")
    navy_blue: QColor = QColor("#000080")
    ncs_red: QColor = QColor("#C40233")
    neon_blue: QColor = QColor("#4D4DFF")
    neon_fuchsia: QColor = QColor("#FE4164")
    neon_green: QColor = QColor("#39FF14")
    neon_orange: QColor = QColor("#FFAD00")
    neon_pink: QColor = QColor("#FF6EC7")
    neon_yellow: QColor = QColor("#CFFF04")
    neutral_black: QColor = QColor("#0B0B0B")
    neutral_gray: QColor = QColor("#898E8C")
    nevada: QColor = QColor("#666A6D")
    new_york_pink: QColor = QColor("#DD8374")
    nickel: QColor = QColor("#727472")
    night: QColor = QColor("#0C090A")
    night_shadow: QColor = QColor("#1C1C1C")
    night_white: QColor = QColor("#E1E1DD")
    non_photo_blue: QColor = QColor("#A4DDED")
    northwestern_purple: QColor = QColor("#4E2A84")
    nude_pink: QColor = QColor("#DDC0B4")
    nyanza: QColor = QColor("#E9FFDB")
    oat: QColor = QColor("#DFC98A")
    obsidian: QColor = QColor("#020403")
    ocean: QColor = QColor("#016064")
    ocean_blue: QColor = QColor("#009DC4")
    ocean_purple: QColor = QColor("#6E2D91")
    ochre: QColor = QColor("#CC7722")
    ochre_orange: QColor = QColor("#C99700")
    ochre_yellow: QColor = QColor("#B5904B")
    off_black: QColor = QColor("#595652")
    office_green: QColor = QColor("#008000")
    oil: QColor = QColor("#3B3131")
    oklahoma_crimson: QColor = QColor("#841617")
    old_burgundy: QColor = QColor("#43302E")
    old_lace: QColor = QColor("#FDF5E6")
    old_lavender: QColor = QColor("#796878")
    old_moss_green: QColor = QColor("#867E36")
    old_rose: QColor = QColor("#C08081")
    old_silver: QColor = QColor("#848482")
    old_wood: QColor = QColor("#F0EEE4")
    old_yellow: QColor = QColor("#C8AF55")
    olive: QColor = QColor("#708238")
    olive_drab: QColor = QColor("#6B8E23")
    olive_drab_7: QColor = QColor("#3C341F")
    olive_gray: QColor = QColor("#A09F8D")
    olive_yellow_ral: QColor = QColor("#9D8F65")
    onyx: QColor = QColor("#353839")
    orange: QColor = QColor("#ED7014")
    orange_pale_orange: QColor = QColor("#FFDFBF")
    orange_paper_orange: QColor = QColor("#FEE8D6")
    orange_peel: QColor = QColor("#FF9F00")
    orange_red: QColor = QColor("#FF4500")
    orange_spice_orange: QColor = QColor("#D76B00")
    orange_white: QColor = QColor("#F8F9F5")
    orange_yellow: QColor = QColor("#FFAD01")
    orchid: QColor = QColor("#DA70D6")
    orioles_orange: QColor = QColor("#FB4F14")
    outer_space: QColor = QColor("#414A4C")
    outrageous_orange: QColor = QColor("#FF6E4A")
    outrageous_orange_red: QColor = QColor("#FF6E4A")
    oxblood_red: QColor = QColor("#800020")
    oxford_blue: QColor = QColor("#002147")
    oxford_gray: QColor = QColor("#8391A1")
    oyster: QColor = QColor("#F0D8D8")
    oyster_white: QColor = QColor("#E3DFD2")
    pacific_blue: QColor = QColor("#1CA9C9")
    pacific_pearl: QColor = QColor("#E8EAE6")
    pakistan_green: QColor = QColor("#006600")
    palatinate: QColor = QColor("#72246C")
    pale_goldenrod: QColor = QColor("#EEE8AA")
    pale_ivory: QColor = QColor("#FDF6E4")
    pale_lime_yellow: QColor = QColor("#DFE69F")
    pale_purple: QColor = QColor("#AA98A9")
    pale_smoke: QColor = QColor("#CAD4D1")
    pale_violet_red: QColor = QColor("#DB7093")
    pale_yellow: QColor = QColor("#FFFFA7")
    panda_black: QColor = QColor("#3C4748")
    pansy_purple: QColor = QColor("#78184A")
    pantone_artichoke_green: QColor = QColor("#4B6F44")
    pantone_blue: QColor = QColor("#0018A8")
    pantone_orange: QColor = QColor("#FE5000")
    paolo_veronese_green: QColor = QColor("#009B7D")
    papaya_whip: QColor = QColor("#FFEFD5")
    paper_white: QColor = QColor("#F7FCFE")
    paradise_pink: QColor = QColor("#E63E62")
    parakeet: QColor = QColor("#03C04A")
    parchment: QColor = QColor("#F1E9D2")
    paris_green: QColor = QColor("#50C878")
    paris_white: QColor = QColor("#C2D8D3")
    parmesan: QColor = QColor("#FDE992")
    passion_pink: QColor = QColor("#CE74A7")
    pastel_black: QColor = QColor("#1D1C1A")
    pastel_gray: QColor = QColor("#CCCDC6")
    pastel_orange: QColor = QColor("#FEBA4F")
    pastel_pink: QColor = QColor("#FFD1DC")
    pastel_red: QColor = QColor("#FF6961")
    pastel_yellow: QColor = QColor("#FFFE71")
    paynes_gray: QColor = QColor("#40404F")
    peach: QColor = QColor("#FAD1AF")
    peach_orange: QColor = QColor("#FFCC99")
    peacock: QColor = QColor("#022D36")
    peacock_blue: QColor = QColor("#005F69")
    peanut_brown: QColor = QColor("#795C34")
    pear: QColor = QColor("#D1E231")
    pearl: QColor = QColor("#FBFCF8")
    pearly_purple: QColor = QColor("#B768A2")
    pebble: QColor = QColor("#333333")
    pecan: QColor = QColor("#4A2511")
    penn_blue: QColor = QColor("#011F5B")
    penn_red: QColor = QColor("#990000")
    penny: QColor = QColor("#532915")
    pepper_orange: QColor = QColor("#E77D22")
    peppermint: QColor = QColor("#F1F9EC")
    perfect_gray: QColor = QColor("#A1A1A1")
    periwinkle: QColor = QColor("#CCCCFF")
    periwinkle_purple: QColor = QColor("#BE93E4")
    persian: QColor = QColor("#CA3433")
    persian_blue: QColor = QColor("#1C39BB")
    persian_green: QColor = QColor("#00A693")
    persian_orange: QColor = QColor("#D99058")
    persian_pink: QColor = QColor("#F77FBE")
    persian_red: QColor = QColor("#CA3433")
    persian_rose_red: QColor = QColor("#FE28A2")
    persimmon: QColor = QColor("#EC5800")
    peru: QColor = QColor("#CD853F")
    pewter: QColor = QColor("#696880")
    philippine_orange: QColor = QColor("#F26E01")
    phillies_red: QColor = QColor("#E81828")
    phlox: QColor = QColor("#DF00FF")
    photon_white: QColor = QColor("#F8F8E8")
    phthalo_blue: QColor = QColor("#000F89")
    phthalo_green: QColor = QColor("#123524")
    pickle: QColor = QColor("#597D35")
    picotee_blue: QColor = QColor("#2E2787")
    piggy_pink: QColor = QColor("#FDDDE6")
    pigment_magenta: QColor = QColor("#FF0090")
    pigment_red: QColor = QColor("#ED1C24")
    pine: QColor = QColor("#234F1E")
    pine_green: QColor = QColor("#01796F")
    pine_needle_color: QColor = QColor("#454D32")
    pink: QColor = QColor("#F699CD")
    pink_gray: QColor = QColor("#E7DADA")
    pink_pearl: QColor = QColor("#E7ACCF")
    pink_salt: QColor = QColor("#F8EEEC")
    pink_sherbet: QColor = QColor("#F78FA7")
    pinterest_red: QColor = QColor("#E60023")
    pistachio: QColor = QColor("#93C572")
    pitch: QColor = QColor("#030001")
    pitch_black: QColor = QColor("#27251F")
    pizza_edge: QColor = QColor("#9A2CA0")
    platinum: QColor = QColor("#E5E4E2")
    plum: QColor = QColor("#DDA0DD")
    polar_bear: QColor = QColor("#EAE9E0")
    polynesian_blue: QColor = QColor("#224C98")
    pomelo_white: QColor = QColor("#F9FFE3")
    pomp_and_power: QColor = QColor("#86608E")
    pompadour: QColor = QColor("#720058")
    popstar_red: QColor = QColor("#BE4F62")
    porcelain: QColor = QColor("#FFFEFC")
    porpoise: QColor = QColor("#4D4D5C")
    portland_orange_red: QColor = QColor("#FF5A36")
    powder: QColor = QColor("#FBFCFA")
    powder_blue: QColor = QColor("#B0E0E6")
    power_black: QColor = QColor("#0E0C01")
    power_gray: QColor = QColor("#707272")
    premium_black: QColor = QColor("#100E09")
    prince_charming: QColor = QColor("#493F5E")
    princeton_orange: QColor = QColor("#FF8F00")
    prismatic_legacy: QColor = QColor("#BA1607")
    prismatic_red: QColor = QColor("#D03D33")
    prismatic_reflections_shade: QColor = QColor("#FF3C28")
    prismatic_vermilion_renewal: QColor = QColor("#CA0123")
    prune_red: QColor = QColor("#701C1C")
    prussian_blue: QColor = QColor("#003153")
    psg_blue: QColor = QColor("#004170")
    psychedelic_yellow: QColor = QColor("#FFF53D")
    puce: QColor = QColor("#CC8899")
    pullman_green: QColor = QColor("#3B331C")
    pumpkin: QColor = QColor("#F5761A")
    pumpkin_orange: QColor = QColor("#FF7518")
    punch: QColor = QColor("#F25278")
    pure_orange: QColor = QColor("#FF8000")
    pure_silver: QColor = QColor("#C3C7C7")
    purple: QColor = QColor("#A32CC4")
    purple_4: QColor = QColor("#551A8B")
    purple_cmyk: QColor = QColor("#6A317F")
    purple_gray: QColor = QColor("#A7A6BA")
    purple_heart: QColor = QColor("#7442C8")
    purple_navy: QColor = QColor("#4E5180")
    purple_pizzazz: QColor = QColor("#FE4EDA")
    purple_taupe: QColor = QColor("#50404D")
    purple_violet: QColor = QColor("#47243C")
    purpureus: QColor = QColor("#9A4EAE")
    queen_blue: QColor = QColor("#436B95")
    radiant_yellow: QColor = QColor("#FC9E21")
    radical_red: QColor = QColor("#FF355E")
    radioactive: QColor = QColor("#FAE500")
    rainbow_yellow: QColor = QColor("#F5DE10")
    raisin: QColor = QColor("#290916")
    raisin_black: QColor = QColor("#242124")
    rajah: QColor = QColor("#FABA5F")
    raspberry: QColor = QColor("#D21F3C")
    raspberry_glace: QColor = QColor("#915F6D")
    raspberry_rose: QColor = QColor("#B3446C")
    raven: QColor = QColor("#050301")
    razzle_dazzle_rose: QColor = QColor("#FF33CC")
    razzmatazz: QColor = QColor("#E3256B")
    razzmic_berry_purple: QColor = QColor("#8D4E85")
    real_gold: QColor = QColor("#E2BF36")
    rebecca_purple: QColor = QColor("#663399")
    red: QColor = QColor("#FF0000")
    red_devil: QColor = QColor("#860111")
    red_fox_orange: QColor = QColor("#985629")
    red_orange_ral: QColor = QColor("#B6481C")
    red_sox: QColor = QColor("#BD3039")
    redwood: QColor = QColor("#A45A52")
    refresh_black: QColor = QColor("#111212")
    refresh_yellow: QColor = QColor("#FFE992")
    regalia_purple: QColor = QColor("#522D80")
    relax_yellow: QColor = QColor("#F3E496")
    reseda_green: QColor = QColor("#6C7C59")
    resolution_blue: QColor = QColor("#002387")
    retro_black: QColor = QColor("#1F201F")
    retro_white: QColor = QColor("#EDEAE2")
    retro_yellow: QColor = QColor("#F0E27B")
    revere_pewter: QColor = QColor("#CCC7B9")
    reversed_gray: QColor = QColor("#080808")
    rhino: QColor = QColor("#B9BBB6")
    rhythm: QColor = QColor("#777696")
    rice: QColor = QColor("#FAF5EF")
    rich_black_fogra29: QColor = QColor("#010B13")
    rich_gray: QColor = QColor("#1F2022")
    rifle_green: QColor = QColor("#444C38")
    ripe_plum: QColor = QColor("#410056")
    road_sign_green: QColor = QColor("#01735C")
    roasted_coffee: QColor = QColor("#4B371C")
    robin_egg_blue: QColor = QColor("#00CCCC")
    rocket_metallic: QColor = QColor("#8A7F8D")
    rose: QColor = QColor("#FC94AF")
    rose_bonbon: QColor = QColor("#F9429E")
    rose_ebony: QColor = QColor("#674846")
    rose_gold: QColor = QColor("#B76E79")
    rose_quartz: QColor = QColor("#AA98A9")
    rose_red: QColor = QColor("#C21E56")
    rose_taupe: QColor = QColor("#905D5D")
    rose_white: QColor = QColor("#FFFAFA")
    rosewood: QColor = QColor("#9E4244")
    rosewood_red: QColor = QColor("#65000B")
    rosy_pink: QColor = QColor("#FF66CC")
    rouge: QColor = QColor("#A94064")
    royal_orange: QColor = QColor("#FF9944")
    royal_periwinkle: QColor = QColor("#BE93D4")
    royal_yellow: QColor = QColor("#FADA5E")
    ruber: QColor = QColor("#CE4676")
    rubine_red: QColor = QColor("#D10056")
    ruby: QColor = QColor("#E0115F")
    ruddy_blue: QColor = QColor("#76ABDF")
    ruddy_pink: QColor = QColor("#E18E96")
    rufous: QColor = QColor("#A81C07")
    rufous_red: QColor = QColor("#A81C07")
    rumba_orange: QColor = QColor("#F06631")
    russet: QColor = QColor("#80461B")
    russet_red: QColor = QColor("#80461B")
    russian_green: QColor = QColor("#679267")
    russian_violet: QColor = QColor("#32174D")
    rust: QColor = QColor("#B7410E")
    rustic_black: QColor = QColor("#1B1A16")
    rustic_gray: QColor = QColor("#A6ABA5")
    rustic_orange: QColor = QColor("#D78C3D")
    rustic_purple: QColor = QColor("#593163")
    rustic_yellow: QColor = QColor("#E2C400")
    rusty_red: QColor = QColor("#DA2C43")
    sable: QColor = QColor("#060606")
    sacramento_green: QColor = QColor("#043927")
    saddle_brown: QColor = QColor("#8B4513")
    safety_orange: QColor = QColor("#FF6700")
    safety_yellow: QColor = QColor("#EED202")
    safflower_orange: QColor = QColor("#FDAE44")
    saffron: QColor = QColor("#F4C430")
    sage: QColor = QColor("#728C69")
    sage_gray: QColor = QColor("#B5BDA5")
    sage_green: QColor = QColor("#9DC183")
    salamander_orange: QColor = QColor("#F05E23")
    salmon: QColor = QColor("#FF9999")
    salmon_orange: QColor = QColor("#FF8066")
    sand: QColor = QColor("#F6D7B0")
    sand_dollar: QColor = QColor("#EDE8BA")
    sandcastle: QColor = QColor("#DAC17C")
    sandstone: QColor = QColor("#D67229")
    sandstorm: QColor = QColor("#ECD540")
    sangria: QColor = QColor("#4D0F28")
    sangria_purple: QColor = QColor("#4D0F28")
    sangria_red: QColor = QColor("#5E1916")
    sanguine_red: QColor = QColor("#BC3F4A")
    sap_green: QColor = QColor("#507D2A")
    sapphire: QColor = QColor("#52B2BF")
    sapphire_blue: QColor = QColor("#0F52BA")
    satin_sheen_gold: QColor = QColor("#CBA135")
    savoy_blue: QColor = QColor("#4B61D1")
    scarlet: QColor = QColor("#FF2400")
    schauss_pink: QColor = QColor("#FF91AF")
    school_bus_yellow: QColor = QColor("#FFD800")
    scorpion: QColor = QColor("#5E5E5E")
    screamin_green: QColor = QColor("#76FF7A")
    sea: QColor = QColor("#F09090")
    sea_gray: QColor = QColor("#9AA2A4")
    sea_green: QColor = QColor("#2E8B57")
    seafoam: QColor = QColor("#3DED97")
    seal: QColor = QColor("#818380")
    seance: QColor = QColor("#61346B")
    seashell: QColor = QColor("#FFF5EE")
    seaweed: QColor = QColor("#354A21")
    selective_yellow: QColor = QColor("#FFBA00")
    sepia: QColor = QColor("#704214")
    shades_on: QColor = QColor("#606060")
    shadow: QColor = QColor("#373737")
    shadow_white: QColor = QColor("#EEF1EA")
    shampoo: QColor = QColor("#FFCFF1")
    shamrock: QColor = QColor("#03AC13")
    shamrock_green: QColor = QColor("#009E60")
    sheen_green: QColor = QColor("#8FD400")
    shimmering_blush: QColor = QColor("#D98695")
    shiny_purple: QColor = QColor("#B941FF")
    shocking_pink: QColor = QColor("#FC0FC0")
    shortbread: QColor = QColor("#FBE790")
    shrek_green: QColor = QColor("#C4D300")
    sienna: QColor = QColor("#A0522D")
    signal_black_ral: QColor = QColor("#2B2B2C")
    signal_yellow: QColor = QColor("#F2A900")
    silk_gray: QColor = QColor("#B6B3A8")
    silver: QColor = QColor("#C0C0C0")
    silver_chalice: QColor = QColor("#ACACAC")
    silver_pink: QColor = QColor("#C4AEAD")
    silvery_pink: QColor = QColor("#DCB5B4")
    simply_white: QColor = QColor("#EFEDE3")
    sky: QColor = QColor("#63C5DA")
    sky_blue: QColor = QColor("#ADD8E6")
    slate: QColor = QColor("#26282A")
    slate_blue: QColor = QColor("#737CA1")
    slate_gray: QColor = QColor("#657383")
    slimy_green: QColor = QColor("#299617")
    smashed_pumpkin: QColor = QColor("#FD673A")
    smitten: QColor = QColor("#C84186")
    smoke: QColor = QColor("#738276")
    smokey_gray: QColor = QColor("#726E6D")
    smoky_black: QColor = QColor("#100C08")
    smyrna_purple: QColor = QColor("#A2627A")
    snapchat_yellow: QColor = QColor("#FFFC00")
    snow: QColor = QColor("#F5FEFD")
    snow_drift: QColor = QColor("#F8FBF8")
    soft_peach: QColor = QColor("#F6F1F4")
    solid_gray: QColor = QColor("#979797")
    solid_yellow: QColor = QColor("#FFE81C")
    sonic_gray: QColor = QColor("#757575")
    soot: QColor = QColor("#160D08")
    space_cadet: QColor = QColor("#1D2951")
    spanish_blue: QColor = QColor("#0070BB")
    spanish_gray: QColor = QColor("#989898")
    spanish_orange: QColor = QColor("#F06105")
    spanish_pink: QColor = QColor("#F7BFBE")
    spanish_red: QColor = QColor("#E60026")
    spanish_violet_purple: QColor = QColor("#4C2882")
    spanish_viridian: QColor = QColor("#007F5C")
    spanish_yellow: QColor = QColor("#F9D73D")
    spatial_white: QColor = QColor("#DEDDDB")
    spectrum_yellow: QColor = QColor("#FCE517")
    spice: QColor = QColor("#7A3803")
    spider: QColor = QColor("#040200")
    spring: QColor = QColor("#F3F0E8")
    spring_bud: QColor = QColor("#A7FC00")
    spring_orange: QColor = QColor("#FD963D")
    spruce: QColor = QColor("#2C3E4C")
    spruce_blue: QColor = QColor("#617178")
    squash: QColor = QColor("#C95B0C")
    squash_orange: QColor = QColor("#E89362")
    steel: QColor = QColor("#4682B4")
    steel_blue: QColor = QColor("#4682B4")
    steel_gray: QColor = QColor("#888B8D")
    steel_pink: QColor = QColor("#CC3366")
    stone: QColor = QColor("#59788E")
    stone_blue: QColor = QColor("#819EA8")
    stone_eagle: QColor = QColor("#D8D9DA")
    stone_gray: QColor = QColor("#928E85")
    stone_white: QColor = QColor("#FBF6F0")
    storm_gray: QColor = QColor("#B5BAB6")
    stormcloud: QColor = QColor("#4F666A")
    stout: QColor = QColor("#0F0B0A")
    strawberry: QColor = QColor("#E8888A")
    strong_magenta: QColor = QColor("#A1045A")
    strong_purple: QColor = QColor("#875692")
    studio: QColor = QColor("#7851A9")
    sugar_cookie: QColor = QColor("#F3EAAF")
    sulfur_yellow: QColor = QColor("#E8DE35")
    sun_yellow_ral: QColor = QColor("#EA9300")
    sunflower: QColor = QColor("#FFDA03")
    sunglow: QColor = QColor("#FFCC33")
    sunrise_orange: QColor = QColor("#E67451")
    sunset_orange: QColor = QColor("#FD5E53")
    sunset_purple: QColor = QColor("#A865B5")
    sunshine_yellow: QColor = QColor("#FFFD37")
    super_pink: QColor = QColor("#CF6BA9")
    swamp: QColor = QColor("#A8C090")
    swan_white: QColor = QColor("#FCFCFC")
    sweet_pink: QColor = QColor("#EE918D")
    syracuse_orange: QColor = QColor("#D44500")
    syrup: QColor = QColor("#481F01")
    tacao: QColor = QColor("#6F3096")
    taffy: QColor = QColor("#FA86C4")
    tall_ships_blue: QColor = QColor("#0E81B9")
    tan: QColor = QColor("#D2B48C")
    tangelo: QColor = QColor("#FC4C02")
    tangerine: QColor = QColor("#F78702")
    tangerine_yellow: QColor = QColor("#FFCC00")
    tango_red: QColor = QColor("#E4717A")
    tap_shoe: QColor = QColor("#2A2B2D")
    taupe: QColor = QColor("#483C32")
    taupe_gray: QColor = QColor("#8B8589")
    tawny: QColor = QColor("#80471C")
    tawny_orange: QColor = QColor("#B25A38")
    tawny_red: QColor = QColor("#CD5700")
    taxi_cab_yellow: QColor = QColor("#FDB813")
    tea_bag: QColor = QColor("#161311")
    tea_green: QColor = QColor("#D0F0C0")
    tea_rose: QColor = QColor("#F4C2C2")
    tea_rose_red: QColor = QColor("#F88379")
    teal: QColor = QColor("#008080")
    tech_black: QColor = QColor("#0D0E0E")
    templeton_gray: QColor = QColor("#788787")
    tender_yellow: QColor = QColor("#EDEEB7")
    tennis_court_green: QColor = QColor("#6C935C")
    terracotta: QColor = QColor("#E2725B")
    terracotta_red: QColor = QColor("#E2725B")
    thistle: QColor = QColor("#D8BFD8")
    thulian_pink: QColor = QColor("#DE6FA1")
    thunder: QColor = QColor("#BDB7AB")
    thyme: QColor = QColor("#5EDC1F")
    tickle_me_pink: QColor = QColor("#FC89AC")
    tiffany_blue: QColor = QColor("#81D8D0")
    tiger: QColor = QColor("#FC6A03")
    tiger_orange: QColor = QColor("#F96815")
    tigers_eye_orange: QColor = QColor("#E08D3C")
    timberwolf: QColor = QColor("#DBD7D2")
    titan_white: QColor = QColor("#EEEEFF")
    titanium_yellow: QColor = QColor("#EEE600")
    tomato: QColor = QColor("#FF6347")
    topaz_orange: QColor = QColor("#FFC87C")
    tornado_cloud: QColor = QColor("#121213")
    tortilla: QColor = QColor("#9A7B4F")
    tottenham_navy: QColor = QColor("#132257")
    tractor_red: QColor = QColor("#FD0E35")
    traditional_purple: QColor = QColor("#8E4585")
    traditional_royal_blue: QColor = QColor("#002366")
    traffic_purple: QColor = QColor("#913073")
    traffic_yellow_ral: QColor = QColor("#EFB700")
    transparent_yellow: QColor = QColor("#F4ECC2")
    tree_sap_orange: QColor = QColor("#CC7711")
    trombone: QColor = QColor("#D2B55B")
    tropical_rainforest: QColor = QColor("#00755E")
    trout: QColor = QColor("#979788")
    true_black: QColor = QColor("#0A0B0B")
    true_blue: QColor = QColor("#2D68C4")
    true_purple: QColor = QColor("#6A0DAD")
    true_silver: QColor = QColor("#8A8D8F")
    trust_yellow: QColor = QColor("#EEBE15")
    tufts_blue: QColor = QColor("#3E8EDE")
    tulip_pink: QColor = QColor("#FF8E8E")
    tumbleweed: QColor = QColor("#DEAA88")
    turkey_red: QColor = QColor("#A91101")
    turkish_blue: QColor = QColor("#4F97A3")
    turkish_rose: QColor = QColor("#B57281")
    turmeric: QColor = QColor("#F0CA28")
    turquoise: QColor = QColor("#40E0D0")
    turquoise_blue: QColor = QColor("#00FFEF")
    turquoise_green: QColor = QColor("#A0D6B4")
    tuscan_brown: QColor = QColor("#6F4E37")
    tuscan_red: QColor = QColor("#7C3030")
    tuscany: QColor = QColor("#FCD12A")
    twilight_lavender_purple: QColor = QColor("#8A496B")
    twitter_blue: QColor = QColor("#1DA1F2")
    tyrian_purple: QColor = QColor("#66023C")
    ucla_blue: QColor = QColor("#2774AE")
    ufo_green: QColor = QColor("#3CD070")
    ultimate_gray: QColor = QColor("#939597")
    ultra_pink: QColor = QColor("#FF6FFF")
    ultra_violet: QColor = QColor("#645394")
    ultramarine_blue: QColor = QColor("#120A8F")
    umber: QColor = QColor("#352315")
    university_of_central_arkansas_purple: QColor = QColor("#4F2D7F")
    university_of_tennessee_orange: QColor = QColor("#F77F00")
    unmellow_yellow: QColor = QColor("#FFFF66")
    unresolved_problem: QColor = QColor("#F3F2ED")
    up_maroon: QColor = QColor("#7B1113")
    upsdell_red: QColor = QColor("#AE2029")
    uranian_blue: QColor = QColor("#AFDBF5")
    urobilin: QColor = QColor("#E1AD21")
    us_air_force_academy_blue: QColor = QColor("#004F98")
    us_flag: QColor = QColor("#BF0A30")
    utah_crimson: QColor = QColor("#D3003F")
    valentine_pink: QColor = QColor("#E6A6BE")
    vampire_black: QColor = QColor("#0F0404")
    vampire_gray: QColor = QColor("#565051")
    vanilla: QColor = QColor("#F3E5AB")
    vanilla_ice: QColor = QColor("#F38FA9")
    venetian_red: QColor = QColor("#C80815")
    verdun: QColor = QColor("#487800")
    vermilion: QColor = QColor("#D74826")
    veronica: QColor = QColor("#A020F0")
    very_light_malachite_green: QColor = QColor("#64E986")
    very_pale_yellow: QColor = QColor("#FFFFBF")
    vibrant_yellow: QColor = QColor("#FFDA29")
    vintage_gray: QColor = QColor("#989692")
    vintage_orange: QColor = QColor("#C9995D")
    vintage_yellow: QColor = QColor("#CBA92B")
    violet: QColor = QColor("#8F00FF")
    violet_blue: QColor = QColor("#324AB2")
    virgin_lust_red: QColor = QColor("#E4181E")
    viridian: QColor = QColor("#609078")
    visa_blue: QColor = QColor("#1A1F71")
    vista_white: QColor = QColor("#FDFCFA")
    vivid_lime_green: QColor = QColor("#A4DE02")
    vivid_orange: QColor = QColor("#FF5E0E")
    vivid_reddish_purple: QColor = QColor("#870074")
    vivid_sky_blue: QColor = QColor("#00CCFF")
    vivid_tangelo: QColor = QColor("#EC7625")
    vivid_white: QColor = QColor("#F8F8F4")
    vivid_yellow: QColor = QColor("#FFE302")
    void: QColor = QColor("#010207")
    volt: QColor = QColor("#CEFF00")
    vulgar_purple_grape_jelly: QColor = QColor("#3E2F84")
    wageningen_green: QColor = QColor("#34B233")
    walnut: QColor = QColor("#432616")
    walnut_hull: QColor = QColor("#1B1813")
    warm_black: QColor = QColor("#004242")
    warm_gray: QColor = QColor("#6E6259")
    warm_yellow: QColor = QColor("#F4E98C")
    warriors_blue: QColor = QColor("#1D428A")
    watermelon: QColor = QColor("#FC6C85")
    web_orange: QColor = QColor("#FFA500")
    west_side_orange: QColor = QColor("#FF9916")
    wet_suit: QColor = QColor("#080706")
    wheat: QColor = QColor("#F5DEB3")
    wheat_orange: QColor = QColor("#F5DEB3")
    white: QColor = QColor("#FFFFFF")
    white_chocolate: QColor = QColor("#EDE6D6")
    white_diamond: QColor = QColor("#E7E8E2")
    white_dove: QColor = QColor("#F0EFE7")
    white_gray: QColor = QColor("#F2F3F5")
    white_heat: QColor = QColor("#FDF9EF")
    white_on_white: QColor = QColor("#EDEFEF")
    white_sand: QColor = QColor("#ECE9E4")
    white_smoke: QColor = QColor("#F5F5F5")
    white_whisper: QColor = QColor("#EAE8E1")
    white_wisp: QColor = QColor("#E9E9E4")
    whitewash: QColor = QColor("#FEFFFC")
    wild_orchid: QColor = QColor("#D470A2")
    wild_strawberry: QColor = QColor("#FF43A4")
    willpower_orange: QColor = QColor("#FD5602")
    wind_cave: QColor = QColor("#1F2024")
    window_gray: QColor = QColor("#989EA1")
    windsor_tan: QColor = QColor("#A75502")
    wine: QColor = QColor("#2C041C")
    wine_dregs_purple: QColor = QColor("#673147")
    wine_red: QColor = QColor("#722F37")
    winter_white: QColor = QColor("#DFDFD5")
    wood: QColor = QColor("#3F301D")
    wood_brown: QColor = QColor("#C19A6B")
    woodsmoke: QColor = QColor("#2B3230")
    x11_gray: QColor = QColor("#BBBCB6")
    xanadu: QColor = QColor("#738678")
    xanthic: QColor = QColor("#EEED09")
    xanthous_orange: QColor = QColor("#F1B42F")
    yacht_club_black: QColor = QColor("#222627")
    yale_blue: QColor = QColor("#00356B")
    yam: QColor = QColor("#CC5801")
    yam_orange: QColor = QColor("#E17E45")
    yankees_blue: QColor = QColor("#0C2340")
    yellow_cmyk: QColor = QColor("#FFFF00")
    yellow_cream: QColor = QColor("#EFDC75")
    yellow_jasmine: QColor = QColor("#CEB00D")
    yellow_orange: QColor = QColor("#D56F00")
    yellow_pantone: QColor = QColor("#FEDF00")
    yellow_rose: QColor = QColor("#FFF000")
    yellow_tan: QColor = QColor("#FFE36E")
    yin_mn_blue: QColor = QColor("#2E5090")
    yolk_yellow_pantone: QColor = QColor("#E2B051")
    young_night: QColor = QColor("#232323")
    your: QColor = QColor("#FFC0C0")
    zinc_yellow_ral: QColor = QColor("#F2CB2E")
    zinnwaldite_brown: QColor = QColor("#2C1608")
    zombie_gray: QColor = QColor("#63645E")

ColorTypeVar: TypeAlias = Union[QColor, QRgba64, Any, 
                                Qt.GlobalColor, Colors, 
                                str, int]


from typing import Iterable, Union
from PySide6.QtCore import QUrl
from PySide6.QtGui import QDesktopServices


class LauncheURL:
    @staticmethod
    def open_url(*urls: list[Union[str, QUrl]]) -> Iterable[bool]:
        '''
        Web URL      : http, https, ftp, etc...
        File URL     : file:///[file path]
        Map Location : geo:[...],[...]
        Phone Number : tel:[phone number]
        Any URL      : [URL...]
        '''
        for url in urls:
            yield QDesktopServices.openUrl(
                QUrl.fromUserInput(url)
                if isinstance(url, str) 
                else url
            )

    ## work on!
    # @staticmethod
    # def open_app(self):
    #     """
    #     Linux: [.desktop, .AppImage, .boundle, .bin, .sh]
    #     MacOS: [.dmg]
    #     Windows: [.msi, .exe]            
    #     """
    #     ...


from enum import Enum


class QPA_Platforms(Enum):
    android: str = "android"
    cocoa: str = "cocoa"
    directfb: str = "directfb"
    eglfs: str = "eglfs"
    ios: str = "ios"
    kms: str = "kms"
    linuxfb: str = "linuxfb"
    minimal: str = "minimal"
    minimalegl: str = "minimalegl"
    offscreen: str = "offscreen"
    openwfd: str = "openwfd"
    qnx: str = "qnx"
    windows: str = "windows"
    wayland: str = "wayland"
    xcb: str = "xcb"


# -*- coding: utf-8 -*-
# This code is based on AjaxTerm/Web-Shell which included a fairly complete
# vt100 implementation as well as a stable process multiplexer.
# I made some small fixes, improved some small parts and added a Session class
# which can be used by the widget.
# License: GPL2
import sys
import os
import fcntl
import array
import threading
import time
import termios
import pty
import signal
import struct
import select
import subprocess

__version__ = "0.3"


class Terminal(object):

    def __init__(self, w, h):
        self.w = w
        self.h = h
        self.vt100_charset_graph = [
            0x25ca, 0x2026, 0x2022, 0x3f,
            0xb6, 0x3f, 0xb0, 0xb1,
            0x3f, 0x3f, 0x2b, 0x2b,
            0x2b, 0x2b, 0x2b, 0xaf,
            0x2014, 0x2014, 0x2014, 0x5f,
            0x2b, 0x2b, 0x2b, 0x2b,
            0x7c, 0x2264, 0x2265, 0xb6,
            0x2260, 0xa3, 0xb7, 0x7f
        ]
        self.vt100_esc = {
            '#8': self.esc_DECALN,
            '(A': self.esc_G0_0,
            '(B': self.esc_G0_1,
            '(0': self.esc_G0_2,
            '(1': self.esc_G0_3,
            '(2': self.esc_G0_4,
            ')A': self.esc_G1_0,
            ')B': self.esc_G1_1,
            ')0': self.esc_G1_2,
            ')1': self.esc_G1_3,
            ')2': self.esc_G1_4,
            '7': self.esc_DECSC,
            '8': self.esc_DECRC,
            '=': self.esc_DECKPAM,
            '>': self.esc_DECKPNM,
            'D': self.esc_IND,
            'E': self.esc_NEL,
            'H': self.esc_HTS,
            'M': self.esc_RI,
            'N': self.esc_SS2,
            'O': self.esc_SS3,
            'P': self.esc_DCS,
            'X': self.esc_SOS,
            'Z': self.esc_DECID,
            '[': self.esc_CSI,
            '\\': self.esc_ST,
            ']': self.esc_OSC,
            '^': self.esc_PM,
            '_': self.esc_APC,
            'c': self.reset_hard,
        }
        self.vt100_csi = {
            '@': self.csi_ICH,
            'A': self.csi_CUU,
            'B': self.csi_CUD,
            'C': self.csi_CUF,
            'D': self.csi_CUB,
            'E': self.csi_CNL,
            'F': self.csi_CPL,
            'G': self.csi_CHA,
            'H': self.csi_CUP,
            'I': self.csi_CHT,
            'J': self.csi_ED,
            'K': self.csi_EL,
            'L': self.csi_IL,
            'M': self.csi_DL,
            'P': self.csi_DCH,
            'S': self.csi_SU,
            'T': self.csi_SD,
            'W': self.csi_CTC,
            'X': self.csi_ECH,
            'Z': self.csi_CBT,
            '`': self.csi_HPA,
            'a': self.csi_HPR,
            'b': self.csi_REP,
            'c': self.csi_DA,
            'd': self.csi_VPA,
            'e': self.csi_VPR,
            'f': self.csi_HVP,
            'g': self.csi_TBC,
            'h': self.csi_SM,
            'l': self.csi_RM,
            'm': self.csi_SGR,
            'n': self.csi_DSR,
            'r': self.csi_DECSTBM,
            's': self.csi_SCP,
            'u': self.csi_RCP,
            'x': self.csi_DECREQTPARM,
            '!p': self.csi_DECSTR,
        }
        self.vt100_keyfilter_ansikeys = {
            '~': '~',
            'A': '\x1b[A',
            'B': '\x1b[B',
            'C': '\x1b[C',
            'D': '\x1b[D',
            'F': '\x1b[F',
            'H': '\x1b[H',
            '1': '\x1b[5~',
            '2': '\x1b[6~',
            '3': '\x1b[2~',
            '4': '\x1b[3~',
            'a': '\x1bOP',
            'b': '\x1bOQ',
            'c': '\x1bOR',
            'd': '\x1bOS',
            'e': '\x1b[15~',
            'f': '\x1b[17~',
            'g': '\x1b[18~',
            'h': '\x1b[19~',
            'i': '\x1b[20~',
            'j': '\x1b[21~',
            'k': '\x1b[23~',
            'l': '\x1b[24~',
        }
        self.vt100_keyfilter_appkeys = {
            '~': '~',
            'A': '\x1bOA',
            'B': '\x1bOB',
            'C': '\x1bOC',
            'D': '\x1bOD',
            'F': '\x1bOF',
            'H': '\x1bOH',
            '1': '\x1b[5~',
            '2': '\x1b[6~',
            '3': '\x1b[2~',
            '4': '\x1b[3~',
            'a': '\x1bOP',
            'b': '\x1bOQ',
            'c': '\x1bOR',
            'd': '\x1bOS',
            'e': '\x1b[15~',
            'f': '\x1b[17~',
            'g': '\x1b[18~',
            'h': '\x1b[19~',
            'i': '\x1b[20~',
            'j': '\x1b[21~',
            'k': '\x1b[23~',
            'l': '\x1b[24~',
        }
        self.reset_hard()

    # Reset functions
    def reset_hard(self):
        # Attribute mask: 0x0XFB0000
        #	X:	Bit 0 - Underlined
        #		Bit 1 - Negative
        #		Bit 2 - Concealed
        #	F:	Foreground
        #	B:	Background
        self.attr = 0x00fe0000
        # UTF-8 decoder
        self.utf8_units_count = 0
        self.utf8_units_received = 0
        self.utf8_char = 0
        # Key filter
        self.vt100_keyfilter_escape = False
        # Last char
        self.vt100_lastchar = 0
        # Control sequences
        self.vt100_parse_len = 0
        self.vt100_parse_state = ""
        self.vt100_parse_func = ""
        self.vt100_parse_param = ""
        # Buffers
        self.vt100_out = ""
        # Invoke other resets
        self.reset_screen()
        self.reset_soft()

    def reset_soft(self):
        # Attribute mask: 0x0XFB0000
        #	X:	Bit 0 - Underlined
        #		Bit 1 - Negative
        #		Bit 2 - Concealed
        #	F:	Foreground
        #	B:	Background
        self.attr = 0x00fe0000
        # Scroll parameters
        self.scroll_area_y0 = 0
        self.scroll_area_y1 = self.h
        # Character sets
        self.vt100_charset_is_single_shift = False
        self.vt100_charset_is_graphical = False
        self.vt100_charset_g_sel = 0
        self.vt100_charset_g = [0, 0]
        # Modes
        self.vt100_mode_insert = False
        self.vt100_mode_lfnewline = False
        self.vt100_mode_cursorkey = False
        self.vt100_mode_column_switch = False
        self.vt100_mode_inverse = False
        self.vt100_mode_origin = False
        self.vt100_mode_autowrap = True
        self.vt100_mode_cursor = True
        self.vt100_mode_alt_screen = False
        self.vt100_mode_backspace = False
        # Init DECSC state
        self.esc_DECSC()
        self.vt100_saved2 = self.vt100_saved
        self.esc_DECSC()

    def reset_screen(self):
        # Screen
        self.screen = array.array('i', [self.attr | 0x20] * self.w * self.h)
        self.screen2 = array.array('i', [self.attr | 0x20] * self.w * self.h)
        # Scroll parameters
        self.scroll_area_y0 = 0
        self.scroll_area_y1 = self.h
        # Cursor position
        self.cx = 0
        self.cy = 0
        # Tab stops
        self.tab_stops = list(range(0, self.w, 8))

    # UTF-8 functions
    def utf8_decode(self, d):
        # o = ''
        # for c in d:
        #    char = ord(c)
        #    if self.utf8_units_count != self.utf8_units_received:
        #        self.utf8_units_received += 1
        #        if (char & 0xc0) == 0x80:
        #            self.utf8_char = (self.utf8_char << 6) | (char & 0x3f)
        #            if self.utf8_units_count == self.utf8_units_received:
        #                if self.utf8_char<0x10000:
        #                    o += chr(self.utf8_char)
        #                self.utf8_units_count = self.utf8_units_received = 0
        #        else:
        #            o += '?'
        #            while self.utf8_units_received:
        #                o += '?'
        #                self.utf8_units_received -= 1
        #            self.utf8_units_count = 0
        #    else:
        #        if (char & 0x80) == 0x00:
        #            o += c
        #        elif (char & 0xe0) == 0xc0:
        #            self.utf8_units_count = 1
        #            self.utf8_char = char & 0x1f
        #        elif (char & 0xf0) == 0xe0:
        #            self.utf8_units_count = 2
        #            self.utf8_char = char & 0x0f
        #        elif (char & 0xf8) == 0xf0:
        #            self.utf8_units_count = 3
        #            self.utf8_char = char & 0x07
        #        else:
        #            o += '?'
        # return o
        return d.decode("utf-8")

    def utf8_charwidth(self, char):
        if char >= 0x2e80:
            return 2
        else:
            return 1

    # Low-level terminal functions
    def peek(self, y0, x0, y1, x1):
        return self.screen[self.w * y0 + x0:self.w * (y1 - 1) + x1]

    def poke(self, y, x, s):
        pos = self.w * y + x
        self.screen[pos:pos + len(s)] = s

    def fill(self, y0, x0, y1, x1, char):
        n = self.w * (y1 - y0 - 1) + (x1 - x0)
        self.poke(y0, x0, array.array('i', [char] * n))

    def clear(self, y0, x0, y1, x1):
        self.fill(y0, x0, y1, x1, self.attr | 0x20)

    # Scrolling functions
    def scroll_area_up(self, y0, y1, n=1):
        n = min(y1 - y0, n)
        self.poke(y0, 0, self.peek(y0 + n, 0, y1, self.w))
        self.clear(y1 - n, 0, y1, self.w)

    def scroll_area_down(self, y0, y1, n=1):
        n = min(y1 - y0, n)
        self.poke(y0 + n, 0, self.peek(y0, 0, y1 - n, self.w))
        self.clear(y0, 0, y0 + n, self.w)

    def scroll_area_set(self, y0, y1):
        y0 = max(0, min(self.h - 1, y0))
        y1 = max(1, min(self.h, y1))
        if y1 > y0:
            self.scroll_area_y0 = y0
            self.scroll_area_y1 = y1

    def scroll_line_right(self, y, x, n=1):
        if x < self.w:
            n = min(self.w - self.cx, n)
            self.poke(y, x + n, self.peek(y, x, y + 1, self.w - n))
            self.clear(y, x, y + 1, x + n)

    def scroll_line_left(self, y, x, n=1):
        if x < self.w:
            n = min(self.w - self.cx, n)
            self.poke(y, x, self.peek(y, x + n, y + 1, self.w))
            self.clear(y, self.w - n, y + 1, self.w)

    # Cursor functions
    def cursor_line_width(self, next_char):
        wx = self.utf8_charwidth(next_char)
        lx = 0
        for x in range(min(self.cx, self.w)):
            char = self.peek(self.cy, x, self.cy + 1, x + 1)[0] & 0xffff
            wx += self.utf8_charwidth(char)
            lx += 1
        return wx, lx

    def cursor_up(self, n=1):
        self.cy = max(self.scroll_area_y0, self.cy - n)

    def cursor_down(self, n=1):
        self.cy = min(self.scroll_area_y1 - 1, self.cy + n)

    def cursor_left(self, n=1):
        self.cx = max(0, self.cx - n)

    def cursor_right(self, n=1):
        self.cx = min(self.w - 1, self.cx + n)

    def cursor_set_x(self, x):
        self.cx = max(0, x)

    def cursor_set_y(self, y):
        self.cy = max(0, min(self.h - 1, y))

    def cursor_set(self, y, x):
        self.cursor_set_x(x)
        self.cursor_set_y(y)

    # Dumb terminal
    def ctrl_BS(self):
        delta_y, cx = divmod(self.cx - 1, self.w)
        cy = max(self.scroll_area_y0, self.cy + delta_y)
        self.cursor_set(cy, cx)

    def ctrl_HT(self, n=1):
        if n > 0 and self.cx >= self.w:
            return
        if n <= 0 and self.cx == 0:
            return
        ts = 0
        for i in range(len(self.tab_stops)):
            if self.cx >= self.tab_stops[i]:
                ts = i
        ts += n
        if len(self.tab_stops) > ts >= 0:
            self.cursor_set_x(self.tab_stops[ts])
        else:
            self.cursor_set_x(self.w - 1)

    def ctrl_LF(self):
        if self.vt100_mode_lfnewline:
            self.ctrl_CR()
        if self.cy == self.scroll_area_y1 - 1:
            self.scroll_area_up(self.scroll_area_y0, self.scroll_area_y1)
        else:
            self.cursor_down()

    def ctrl_CR(self):
        self.cursor_set_x(0)

    def dumb_write(self, char):
        if char < 32:
            if char == 8:
                self.ctrl_BS()
            elif char == 9:
                self.ctrl_HT()
            elif 10 <= char <= 12:
                self.ctrl_LF()
            elif char == 13:
                self.ctrl_CR()
            return True
        return False

    def dumb_echo(self, char):
        # Check right bound
        wx, cx = self.cursor_line_width(char)
        # Newline
        if wx > self.w:
            if self.vt100_mode_autowrap:
                self.ctrl_CR()
                self.ctrl_LF()
            else:
                self.cx = cx - 1
        if self.vt100_mode_insert:
            self.scroll_line_right(self.cy, self.cx)
        if self.vt100_charset_is_single_shift:
            self.vt100_charset_is_single_shift = False
        elif self.vt100_charset_is_graphical and (char & 0xffe0) == 0x0060:
            char = self.vt100_charset_graph[char - 0x60]
        self.poke(self.cy, self.cx, array.array('i', [self.attr | char]))
        self.cursor_set_x(self.cx + 1)

    # VT100 CTRL, ESC, CSI handlers
    def vt100_charset_update(self):
        self.vt100_charset_is_graphical = (
                self.vt100_charset_g[self.vt100_charset_g_sel] == 2)

    def vt100_charset_set(self, g):
        # Invoke active character set
        self.vt100_charset_g_sel = g
        self.vt100_charset_update()

    def vt100_charset_select(self, g, charset):
        # Select charset
        self.vt100_charset_g[g] = charset
        self.vt100_charset_update()

    def vt100_setmode(self, p, state):
        # Set VT100 mode
        p = self.vt100_parse_params(p, [], False)
        for m in p:
            if m == '4':
                # Insertion replacement mode
                self.vt100_mode_insert = state
            elif m == '20':
                # Linefeed/new line mode
                self.vt100_mode_lfnewline = state
            elif m == '?1':
                # Cursor key mode
                self.vt100_mode_cursorkey = state
            elif m == '?3':
                # Column mode
                if self.vt100_mode_column_switch:
                    if state:
                        self.w = 132
                    else:
                        self.w = 80
                    self.reset_screen()
            elif m == '?5':
                # Screen mode
                self.vt100_mode_inverse = state
            elif m == '?6':
                # Region origin mode
                self.vt100_mode_origin = state
                if state:
                    self.cursor_set(self.scroll_area_y0, 0)
                else:
                    self.cursor_set(0, 0)
            elif m == '?7':
                # Autowrap mode
                self.vt100_mode_autowrap = state
            elif m == '?25':
                # Text cursor enable mode
                self.vt100_mode_cursor = state
            elif m == '?40':
                # Column switch control
                self.vt100_mode_column_switch = state
            elif m == '?47':
                # Alternate screen mode
                if ((state and not self.vt100_mode_alt_screen) or
                        (not state and self.vt100_mode_alt_screen)):
                    self.screen, self.screen2 = self.screen2, self.screen
                    self.vt100_saved, self.vt100_saved2 = self.vt100_saved2, self.vt100_saved
                self.vt100_mode_alt_screen = state
            elif m == '?67':
                # Backspace/delete
                self.vt100_mode_backspace = state

    def ctrl_SO(self):
        # Shift out
        self.vt100_charset_set(1)

    def ctrl_SI(self):
        # Shift in
        self.vt100_charset_set(0)

    def esc_CSI(self):
        # CSI start sequence
        self.vt100_parse_reset('csi')

    def esc_DECALN(self):
        # Screen alignment display
        self.fill(0, 0, self.h, self.w, 0x00fe0045)

    def esc_G0_0(self):
        self.vt100_charset_select(0, 0)

    def esc_G0_1(self):
        self.vt100_charset_select(0, 1)

    def esc_G0_2(self):
        self.vt100_charset_select(0, 2)

    def esc_G0_3(self):
        self.vt100_charset_select(0, 3)

    def esc_G0_4(self):
        self.vt100_charset_select(0, 4)

    def esc_G1_0(self):
        self.vt100_charset_select(1, 0)

    def esc_G1_1(self):
        self.vt100_charset_select(1, 1)

    def esc_G1_2(self):
        self.vt100_charset_select(1, 2)

    def esc_G1_3(self):
        self.vt100_charset_select(1, 3)

    def esc_G1_4(self):
        self.vt100_charset_select(1, 4)

    def esc_DECSC(self):
        # Store cursor
        self.vt100_saved = {}
        self.vt100_saved['cx'] = self.cx
        self.vt100_saved['cy'] = self.cy
        self.vt100_saved['attr'] = self.attr
        self.vt100_saved['charset_g_sel'] = self.vt100_charset_g_sel
        self.vt100_saved['charset_g'] = self.vt100_charset_g[:]
        self.vt100_saved['mode_autowrap'] = self.vt100_mode_autowrap
        self.vt100_saved['mode_origin'] = self.vt100_mode_origin

    def esc_DECRC(self):
        # Retore cursor
        self.cx = self.vt100_saved['cx']
        self.cy = self.vt100_saved['cy']
        self.attr = self.vt100_saved['attr']
        self.vt100_charset_g_sel = self.vt100_saved['charset_g_sel']
        self.vt100_charset_g = self.vt100_saved['charset_g'][:]
        self.vt100_charset_update()
        self.vt100_mode_autowrap = self.vt100_saved['mode_autowrap']
        self.vt100_mode_origin = self.vt100_saved['mode_origin']

    def esc_DECKPAM(self):
        # Application keypad mode
        pass

    def esc_DECKPNM(self):
        # Numeric keypad mode
        pass

    def esc_IND(self):
        # Index
        self.ctrl_LF()

    def esc_NEL(self):
        # Next line
        self.ctrl_CR()
        self.ctrl_LF()

    def esc_HTS(self):
        # Character tabulation set
        self.csi_CTC('0')

    def esc_RI(self):
        # Reverse line feed
        if self.cy == self.scroll_area_y0:
            self.scroll_area_down(self.scroll_area_y0, self.scroll_area_y1)
        else:
            self.cursor_up()

    def esc_SS2(self):
        # Single-shift two
        self.vt100_charset_is_single_shift = True

    def esc_SS3(self):
        # Single-shift three
        self.vt100_charset_is_single_shift = True

    def esc_DCS(self):
        # Device control string
        self.vt100_parse_reset('str')

    def esc_SOS(self):
        # Start of string
        self.vt100_parse_reset('str')

    def esc_DECID(self):
        # Identify terminal
        self.csi_DA('0')

    def esc_ST(self):
        # String terminator
        pass

    def esc_OSC(self):
        # Operating system command
        self.vt100_parse_reset('str')

    def esc_PM(self):
        # Privacy message
        self.vt100_parse_reset('str')

    def esc_APC(self):
        # Application program command
        self.vt100_parse_reset('str')

    def csi_ICH(self, p):
        # Insert character
        p = self.vt100_parse_params(p, [1])
        self.scroll_line_right(self.cy, self.cx, p[0])

    def csi_CUU(self, p):
        # Cursor up
        p = self.vt100_parse_params(p, [1])
        self.cursor_up(max(1, p[0]))

    def csi_CUD(self, p):
        # Cursor down
        p = self.vt100_parse_params(p, [1])
        self.cursor_down(max(1, p[0]))

    def csi_CUF(self, p):
        # Cursor right
        p = self.vt100_parse_params(p, [1])
        self.cursor_right(max(1, p[0]))

    def csi_CUB(self, p):
        # Cursor left
        p = self.vt100_parse_params(p, [1])
        self.cursor_left(max(1, p[0]))

    def csi_CNL(self, p):
        # Cursor next line
        self.csi_CUD(p)
        self.ctrl_CR()

    def csi_CPL(self, p):
        # Cursor preceding line
        self.csi_CUU(p)
        self.ctrl_CR()

    def csi_CHA(self, p):
        # Cursor character absolute
        p = self.vt100_parse_params(p, [1])
        self.cursor_set_x(p[0] - 1)

    def csi_CUP(self, p):
        # Set cursor position
        p = self.vt100_parse_params(p, [1, 1])
        if self.vt100_mode_origin:
            self.cursor_set(self.scroll_area_y0 + p[0] - 1, p[1] - 1)
        else:
            self.cursor_set(p[0] - 1, p[1] - 1)

    def csi_CHT(self, p):
        # Cursor forward tabulation
        p = self.vt100_parse_params(p, [1])
        self.ctrl_HT(max(1, p[0]))

    def csi_ED(self, p):
        # Erase in display
        p = self.vt100_parse_params(p, ['0'], False)
        if p[0] == '0':
            self.clear(self.cy, self.cx, self.h, self.w)
        elif p[0] == '1':
            self.clear(0, 0, self.cy + 1, self.cx + 1)
        elif p[0] == '2':
            self.clear(0, 0, self.h, self.w)

    def csi_EL(self, p):
        # Erase in line
        p = self.vt100_parse_params(p, ['0'], False)
        if p[0] == '0':
            self.clear(self.cy, self.cx, self.cy + 1, self.w)
        elif p[0] == '1':
            self.clear(self.cy, 0, self.cy + 1, self.cx + 1)
        elif p[0] == '2':
            self.clear(self.cy, 0, self.cy + 1, self.w)

    def csi_IL(self, p):
        # Insert line
        p = self.vt100_parse_params(p, [1])
        if self.cy >= self.scroll_area_y0 and self.cy < self.scroll_area_y1:
            self.scroll_area_down(self.cy, self.scroll_area_y1, max(1, p[0]))

    def csi_DL(self, p):
        # Delete line
        p = self.vt100_parse_params(p, [1])
        if self.scroll_area_y0 <= self.cy < self.scroll_area_y1:
            self.scroll_area_up(self.cy, self.scroll_area_y1, max(1, p[0]))

    def csi_DCH(self, p):
        # Delete characters
        p = self.vt100_parse_params(p, [1])
        self.scroll_line_left(self.cy, self.cx, max(1, p[0]))

    def csi_SU(self, p):
        # Scroll up
        p = self.vt100_parse_params(p, [1])
        self.scroll_area_up(self.scroll_area_y0, self.scroll_area_y1, max(1, p[0]))

    def csi_SD(self, p):
        # Scroll down
        p = self.vt100_parse_params(p, [1])
        self.scroll_area_down(self.scroll_area_y0, self.scroll_area_y1, max(1, p[0]))

    def csi_CTC(self, p):
        # Cursor tabulation control
        p = self.vt100_parse_params(p, ['0'], False)
        for m in p:
            if m == '0':
                try:
                    ts = self.tab_stops.index(self.cx)
                except ValueError:
                    tab_stops = self.tab_stops
                    tab_stops.append(self.cx)
                    tab_stops.sort()
                    self.tab_stops = tab_stops
            elif m == '2':
                try:
                    self.tab_stops.remove(self.cx)
                except ValueError:
                    pass
            elif m == '5':
                self.tab_stops = [0]

    def csi_ECH(self, p):
        # Erase character
        p = self.vt100_parse_params(p, [1])
        n = min(self.w - self.cx, max(1, p[0]))
        self.clear(self.cy, self.cx, self.cy + 1, self.cx + n)

    def csi_CBT(self, p):
        # Cursor backward tabulation
        p = self.vt100_parse_params(p, [1])
        self.ctrl_HT(1 - max(1, p[0]))

    def csi_HPA(self, p):
        # Character position absolute
        p = self.vt100_parse_params(p, [1])
        self.cursor_set_x(p[0] - 1)

    def csi_HPR(self, p):
        # Character position forward
        self.csi_CUF(p)

    def csi_REP(self, p):
        # Repeat
        p = self.vt100_parse_params(p, [1])
        if self.vt100_lastchar < 32:
            return
        n = min(2000, max(1, p[0]))
        while n:
            self.dumb_echo(self.vt100_lastchar)
            n -= 1
        self.vt100_lastchar = 0

    def csi_DA(self, p):
        # Device attributes
        p = self.vt100_parse_params(p, ['0'], False)
        if p[0] == '0':
            self.vt100_out = "\x1b[?1;2c"
        elif p[0] == '>0' or p[0] == '>':
            self.vt100_out = "\x1b[>0;184;0c"

    def csi_VPA(self, p):
        # Line position absolute
        p = self.vt100_parse_params(p, [1])
        self.cursor_set_y(p[0] - 1)

    def csi_VPR(self, p):
        # Line position forward
        self.csi_CUD(p)

    def csi_HVP(self, p):
        # Character and line position
        self.csi_CUP(p)

    def csi_TBC(self, p):
        # Tabulation clear
        p = self.vt100_parse_params(p, ['0'], False)
        if p[0] == '0':
            self.csi_CTC('2')
        elif p[0] == '3':
            self.csi_CTC('5')

    def csi_SM(self, p):
        # Set mode
        self.vt100_setmode(p, True)

    def csi_RM(self, p):
        # Reset mode
        self.vt100_setmode(p, False)

    def csi_SGR(self, p):
        # Select graphic rendition
        p = self.vt100_parse_params(p, [0])
        for m in p:
            if m == 0:
                # Reset
                self.attr = 0x00fe0000
            elif m == 4:
                # Underlined
                self.attr |= 0x01000000
            elif m == 7:
                # Negative
                self.attr |= 0x02000000
            elif m == 8:
                # Concealed
                self.attr |= 0x04000000
            elif m == 24:
                # Not underlined
                self.attr &= 0x7eff0000
            elif m == 27:
                # Positive
                self.attr &= 0x7dff0000
            elif m == 28:
                # Revealed
                self.attr &= 0x7bff0000
            elif 30 <= m <= 37:
                # Foreground
                self.attr = (self.attr & 0x7f0f0000) | ((m - 30) << 20)
            elif m == 39:
                # Default fg color
                self.attr = (self.attr & 0x7f0f0000) | 0x00f00000
            elif 40 <= m <= 47:
                # Background
                self.attr = (self.attr & 0x7ff00000) | ((m - 40) << 16)
            elif m == 49:
                # Default bg color
                self.attr = (self.attr & 0x7ff00000) | 0x000e0000

    def csi_DSR(self, p):
        # Device status report
        p = self.vt100_parse_params(p, ['0'], False)
        if p[0] == '5':
            self.vt100_out = "\x1b[0n"
        elif p[0] == '6':
            x = self.cx + 1
            y = self.cy + 1
            self.vt100_out = '\x1b[%d;%dR' % (y, x)
        elif p[0] == '7':
            self.vt100_out = 'WebShell'
        elif p[0] == '8':
            self.vt100_out = __version__
        elif p[0] == '?6':
            x = self.cx + 1
            y = self.cy + 1
            self.vt100_out = '\x1b[?%d;%dR' % (y, x)
        elif p[0] == '?15':
            self.vt100_out = '\x1b[?13n'
        elif p[0] == '?25':
            self.vt100_out = '\x1b[?20n'
        elif p[0] == '?26':
            self.vt100_out = '\x1b[?27;1n'
        elif p[0] == '?53':
            self.vt100_out = '\x1b[?53n'

    def csi_DECSTBM(self, p):
        # Set top and bottom margins
        p = self.vt100_parse_params(p, [1, self.h])
        self.scroll_area_set(p[0] - 1, p[1])
        if self.vt100_mode_origin:
            self.cursor_set(self.scroll_area_y0, 0)
        else:
            self.cursor_set(0, 0)

    def csi_SCP(self, p):
        # Save cursor position
        self.vt100_saved_cx = self.cx
        self.vt100_saved_cy = self.cy

    def csi_RCP(self, p):
        # Restore cursor position
        self.cx = self.vt100_saved_cx
        self.cy = self.vt100_saved_cy

    def csi_DECREQTPARM(self, p):
        # Request terminal parameters
        p = self.vt100_parse_params(p, [], False)
        if p[0] == '0':
            self.vt100_out = "\x1b[2;1;1;112;112;1;0x"
        elif p[0] == '1':
            self.vt100_out = "\x1b[3;1;1;112;112;1;0x"

    def csi_DECSTR(self, p):
        # Soft terminal reset
        self.reset_soft()

    # VT100 Parser
    def vt100_parse_params(self, p, d, to_int=True):
        # Process parameters (params p with defaults d)
        # Add prefix to all parameters
        prefix = ''
        if len(p) > 0:
            if '<' <= p[0] <= '?':
                prefix = p[0]
                p = p[1:]
            p = p.split(';')
        else:
            p = ''
        # Process parameters
        n = max(len(p), len(d))
        o = []
        for i in range(n):
            value_def = False
            if i < len(p):
                value = prefix + p[i]
                value_def = True
                if to_int:
                    try:
                        value = int(value)
                    except ValueError:
                        value_def = False
            if (not value_def) and i < len(d):
                value = d[i]
            o.append(value)
        return o

    def vt100_parse_reset(self, vt100_parse_state=""):
        self.vt100_parse_state = vt100_parse_state
        self.vt100_parse_len = 0
        self.vt100_parse_func = ""
        self.vt100_parse_param = ""

    def vt100_parse_process(self):
        if self.vt100_parse_state == 'esc':
            # ESC mode
            f = self.vt100_parse_func
            try:
                self.vt100_esc[f]()
            except KeyError:
                pass
            if self.vt100_parse_state == 'esc':
                self.vt100_parse_reset()
        else:
            # CSI mode
            f = self.vt100_parse_func
            p = self.vt100_parse_param
            try:
                self.vt100_csi[f](p)
            except KeyError:
                pass
            if self.vt100_parse_state == 'csi':
                self.vt100_parse_reset()

    def vt100_write(self, char):
        if char < 32:
            if char == 27:
                self.vt100_parse_reset('esc')
                return True
            elif char == 14:
                self.ctrl_SO()
            elif char == 15:
                self.ctrl_SI()
        elif (char & 0xffe0) == 0x0080:
            self.vt100_parse_reset('esc')
            self.vt100_parse_func = chr(char - 0x40)
            self.vt100_parse_process()
            return True

        if self.vt100_parse_state:
            if self.vt100_parse_state == 'str':
                if char >= 32:
                    return True
                self.vt100_parse_reset()
            else:
                if char < 32:
                    if char == 24 or char == 26:
                        self.vt100_parse_reset()
                        return True
                else:
                    self.vt100_parse_len += 1
                    if self.vt100_parse_len > 32:
                        self.vt100_parse_reset()
                    else:
                        char_msb = char & 0xf0
                        if char_msb == 0x20:
                            # Intermediate bytes (added to function)
                            self.vt100_parse_func += chr(char)
                        elif char_msb == 0x30 and self.vt100_parse_state == 'csi':
                            # Parameter byte
                            self.vt100_parse_param += chr(char)
                        else:
                            # Function byte
                            self.vt100_parse_func += chr(char)
                            self.vt100_parse_process()
                        return True
        self.vt100_lastchar = char
        return False

    # External interface
    def set_size(self, w, h):
        if w < 2 or w > 256 or h < 2 or h > 256:
            return False
        self.w = w
        self.h = h
        self.reset_screen()
        return True

    def read(self):
        d = self.vt100_out
        self.vt100_out = ""
        return d

    def write(self, d):
        d = self.utf8_decode(d)
        # d = d.decode("utf-8")
        for c in d:
            char = ord(c)
            if self.vt100_write(char):
                continue
            if self.dumb_write(char):
                continue
            if char <= 0xffff:
                self.dumb_echo(char)
        return True

    def pipe(self, d):
        o = ''
        d = self.utf8_decode(d)
        for c in d:
            char = ord(c)
            if self.vt100_keyfilter_escape:
                self.vt100_keyfilter_escape = False
                try:
                    if self.vt100_mode_cursorkey:
                        o += self.vt100_keyfilter_appkeys[c]
                    else:
                        o += self.vt100_keyfilter_ansikeys[c]
                except KeyError:
                    pass
            elif c == '~':
                self.vt100_keyfilter_escape = True
            elif char == 127:
                if self.vt100_mode_backspace:
                    o += chr(8)
                else:
                    o += chr(127)
            else:
                o += c
                if self.vt100_mode_lfnewline and char == 13:
                    o += chr(10)
        return o

    def dump(self):
        screen = []
        attr_ = -1
        cx, cy = min(self.cx, self.w - 1), self.cy
        for y in range(0, self.h):
            wx = 0
            line = [""]
            for x in range(0, self.w):
                d = self.screen[y * self.w + x]
                char = d & 0xffff
                attr = d >> 16
                # Cursor
                if cy == y and cx == x and self.vt100_mode_cursor:
                    attr = attr & 0xfff0 | 0x000c
                # Attributes
                if attr != attr_:
                    if attr_ != -1:
                        line.append("")
                    bg = attr & 0x000f
                    fg = (attr & 0x00f0) >> 4
                    # Inverse
                    inv = attr & 0x0200
                    inv2 = self.vt100_mode_inverse
                    if (inv and not inv2) or (inv2 and not inv):
                        fg, bg = bg, fg
                    # Concealed
                    if attr & 0x0400:
                        fg = 0xc
                    # Underline
                    if attr & 0x0100:
                        ul = True
                    else:
                        ul = False
                    line.append((fg, bg, ul))
                    line.append("")
                    attr_ = attr
                wx += self.utf8_charwidth(char)
                if wx <= self.w:
                    line[-1] += chr(char)
            screen.append(line)

        return (cx, cy), screen


def synchronized(func):
    def wrapper(self, *args, **kwargs):
        try:
            self.lock.acquire()
        except AttributeError:
            self.lock = threading.RLock()
            self.lock.acquire()
        try:
            result = func(self, *args, **kwargs)
        finally:
            self.lock.release()
        return result

    return wrapper


class Multiplexer(object):

    def __init__(self, cmd="/bin/bash", env_term="xterm-color", timeout=60 * 60 * 24):
        # Set Linux signal handler
        if sys.platform in ("linux2", "linux3"):
            self.sigchldhandler = signal.signal(signal.SIGCHLD, signal.SIG_IGN)
        # Session
        self.session = {}
        self.cmd = cmd
        self.env_term = env_term
        self.timeout = timeout

        # Supervisor thread
        self.signal_stop = 0
        self.thread = threading.Thread(target=self.proc_thread)
        self.thread.start()

    def stop(self):
        # Stop supervisor thread
        self.signal_stop = 1
        self.thread.join()

    def proc_resize(self, sid, w, h):
        fd = self.session[sid]['fd']
        # Set terminal size
        try:
            fcntl.ioctl(fd,
                        struct.unpack('i',
                                      struct.pack('I', termios.TIOCSWINSZ)
                                      )[0],
                        struct.pack("HHHH", h, w, 0, 0))
        except (IOError, OSError):
            pass
        self.session[sid]['term'].set_size(w, h)
        self.session[sid]['w'] = w
        self.session[sid]['h'] = h

    @synchronized
    def proc_keepalive(self, sid, w, h, cmd=None):
        if not sid in self.session:
            # Start a new session
            self.session[sid] = {
                'state': 'unborn',
                'term': Terminal(w, h),
                'time': time.time(),
                'w': w,
                'h': h}
            return self.proc_spawn(sid, cmd)
        elif self.session[sid]['state'] == 'alive':
            self.session[sid]['time'] = time.time()
            # Update terminal size
            if self.session[sid]['w'] != w or self.session[sid]['h'] != h:
                self.proc_resize(sid, w, h)
            return True
        else:
            return False

    def proc_spawn(self, sid, cmd=None):
        # Session
        self.session[sid]['state'] = 'alive'
        w, h = self.session[sid]['w'], self.session[sid]['h']
        # Fork new process
        try:
            pid, fd = pty.fork()
        except (IOError, OSError):
            self.session[sid]['state'] = 'dead'
            return False
        if pid == 0:
            cmd = cmd or self.cmd
            # Safe way to make it work under BSD and Linux
            try:
                ls = os.environ['LANG'].split('.')
            except KeyError:
                ls = []
            if len(ls) < 2:
                ls = ['en_US', 'UTF-8']
            try:
                os.putenv('COLUMNS', str(w))
                os.putenv('LINES', str(h))
                os.putenv('TERM', self.env_term)
                os.putenv('PATH', os.environ['PATH'])
                os.putenv('LANG', ls[0] + '.UTF-8')
                # os.system(cmd)
                p = subprocess.Popen(cmd, shell=False)
                # print "called with subprocess", p.pid
                child_pid, sts = os.waitpid(p.pid, 0)
                # print "child_pid", child_pid, sts
            except (IOError, OSError):
                pass
            # self.proc_finish(sid)
            os._exit(0)
        else:
            # Store session vars
            self.session[sid]['pid'] = pid
            self.session[sid]['fd'] = fd
            # Set file control
            fcntl.fcntl(fd, fcntl.F_SETFL, os.O_NONBLOCK)
            # Set terminal size
            self.proc_resize(sid, w, h)
            return True

    def proc_waitfordeath(self, sid):
        try:
            os.close(self.session[sid]['fd'])
        except (KeyError, IOError, OSError):
            pass
        if sid in self.session:
            if 'fd' in self.session[sid]:
                del self.session[sid]['fd']
        try:
            os.waitpid(self.session[sid]['pid'], 0)
        except (KeyError, IOError, OSError):
            pass
        if sid in self.session:
            if 'pid' in self.session[sid]:
                del self.session[sid]['pid']
        self.session[sid]['state'] = 'dead'
        return True

    def proc_bury(self, sid):
        if self.session[sid]['state'] == 'alive':
            try:
                os.kill(self.session[sid]['pid'], signal.SIGTERM)
            except (IOError, OSError):
                pass
        self.proc_waitfordeath(sid)
        if sid in self.session:
            del self.session[sid]
        return True

    @synchronized
    def proc_buryall(self):
        for sid in list(self.session.keys()):
            self.proc_bury(sid)

    @synchronized
    def proc_read(self, sid):
        """
        Read from process
        """
        if sid not in self.session:
            return False
        elif self.session[sid]['state'] != 'alive':
            return False
        try:
            fd = self.session[sid]['fd']
            d = os.read(fd, 65536)
            if not d:
                # Process finished, BSD
                self.proc_waitfordeath(sid)
                return False
        except (IOError, OSError):
            # Process finished, Linux
            self.proc_waitfordeath(sid)
            return False
        term = self.session[sid]['term']
        term.write(d)
        # Read terminal response
        d = term.read()
        if d:
            try:
                os.write(fd, d)
            except (IOError, OSError):
                return False
        return True

    @synchronized
    def proc_write(self, sid, d):
        """
        Write to process
        """
        if sid not in self.session:
            return False
        elif self.session[sid]['state'] != 'alive':
            return False
        try:
            term = self.session[sid]['term']
            d = term.pipe(d)
            fd = self.session[sid]['fd']
            os.write(fd, d.encode("utf-8"))
        except (IOError, OSError):
            return False
        return True

    @synchronized
    def proc_dump(self, sid):
        """
        Dump terminal output
        """
        if sid not in self.session:
            return False
        return self.session[sid]['term'].dump()

    @synchronized
    def proc_getalive(self):
        """
        Get alive sessions, bury timed out ones
        """
        fds = []
        fd2sid = {}
        now = time.time()
        for sid in list(self.session.keys()):
            then = self.session[sid]['time']
            if (now - then) > self.timeout:
                self.proc_bury(sid)
            else:
                if self.session[sid]['state'] == 'alive':
                    fds.append(self.session[sid]['fd'])
                    fd2sid[self.session[sid]['fd']] = sid
        return fds, fd2sid

    def proc_thread(self):
        """
        Supervisor thread
        """
        while not self.signal_stop:
            # Read fds
            (fds, fd2sid) = self.proc_getalive()
            try:
                i, o, e = select.select(fds, [], [], 1.0)
            except (IOError, OSError):
                i = []
            for fd in i:
                sid = fd2sid[fd]
                self.proc_read(sid)
                self.session[sid]["changed"] = time.time()
            if len(i):
                time.sleep(0.002)
        self.proc_buryall()


def ssh_command(login, executable="ssh"):
    cmd = executable
    cmd += ' -oPreferredAuthentications=keyboard-interactive,password'
    cmd += ' -oNoHostAuthenticationForLocalhost=yes'
    cmd += ' -oLogLevel=FATAL'
    cmd += ' -F/dev/null -l' + login + ' localhost'
    return cmd


class Session(object):
    _mux = None

    @classmethod
    def close_all(cls):
        Session._mux.stop()

    def __init__(self, cmd=None, width=80, height=24):
        if not Session._mux:
            Session._mux = Multiplexer()
        self._session_id = "%s-%s" % (time.time(), id(self))
        self._width = width
        self._height = height
        self._started = False

    def resize(self, width, height):
        self._width = width
        self._height = height
        if self._started:
            self.keepalive()

    def start(self, cmd=None):
        self._started = Session._mux.proc_keepalive(
            self._session_id, self._width, self._height, cmd or self.cmd)
        return self._started

    def close(self):
        return Session._mux.proc_bury(self._session_id)

    stop = close

    def is_alive(self):
        return Session._mux.session.get(self._session_id, {}).get('state') == 'alive'

    def keepalive(self):
        return Session._mux.proc_keepalive(self._session_id, self._width, self._height)

    def dump(self):
        if self.keepalive():
            return Session._mux.proc_dump(self._session_id)

    def write(self, data):
        if self.keepalive():
            Session._mux.proc_write(self._session_id, data)

    def last_change(self):
        return Session._mux.session.get(self._session_id, {}).get("changed", None)

    def pid(self):
        return Session._mux.session.get(self._session_id, {}).get("pid", None)


if __name__ == "__main__":
    w, h = (80, 24)
    cmd = "/bin/ls --color=yes"
    multiplex = Multiplexer(cmd)
    sid = "session-id-%s"
    if multiplex.proc_keepalive(sid, w, h):
        # multiplex.proc_write(sid, k)
        time.sleep(1)
        # print multiplex.proc_dump(sid)
        print("Output:", multiplex.proc_dump(sid))
    multiplex.stop()



#!/usr/bin/python3
# coding:utf-8

from PySide6.QtGui import QColor
from random import random
from PIL import Image
from re import split


def get_image_average_color(image: str, 
                            size: tuple = (100, 100)) -> list[int]:
    img = Image.open(image)
    img = img.convert("RGB")
    img = img.resize(size)
    return img.getpixel(tuple(map(lambda x: x  // 2, size)))


def mixColor(c1: QColor, c2: QColor, weight: float) -> QColor:
    """ mix two color

    Parameters
    ----------
    c1, c2: QColor
        the color to be mixed

    weight: float
        the weight of first color
    """
    r = int(c1.red()*weight + c2.red()*(1-weight))
    g = int(c1.green()*weight + c2.green()*(1-weight))
    b = int(c1.blue()*weight + c2.blue()*(1-weight))
    return QColor(r, g, b)


def mixLight(color: QColor, weight: float) -> QColor:
    """ mix color with white

    Parameters
    ----------
    color: QColor
        the color to be mixed

    weight: float
        the weight of `color`
    """
    return mixColor(color, QColor(255, 255, 255), weight)

def mixDark(color: QColor, weight: float) -> QColor:
    """ mix color with black

    Parameters
    ----------
    color: QColor
        the color to be mixed

    weight: float
        the weight of `color`
    """
    return mixColor(color, QColor(0, 0, 0), weight)


def translucent(color: QColor, alpha: int) -> QColor:
    return QColor(color.red(), color.green(), color.blue(), alpha)


def get_color_from_hex(color: str):
    '''Transform a hex string color to a kivy
    :class:`~kivy.graphics.Color`.
    '''

    if color.startswith('#'):
        return get_color_from_hex(color[1:])

    value = [int(x, 16) / 255.
             for x in split('([0-9a-f]{2})', color.lower()) 
             if x != '']

    if len(value) == 3:
        value.append(1.0)

    return value


def get_hex_from_color(*rgb: list[int]) -> str:
    '''Transform a rgb(0, 0, 0) color to hex value::
        >>> get_hex_from_color((0, 1, 0))
        '#00ff00'
        >>> get_hex_from_color((25, 77, 90, 5))
        '#3fc4e57f'

    '''
    return '#' + ''.join(['{0:02x}'.format(int(x)) for x in rgb])


def get_random_color(alpha: float = 1.0) -> list[int]:
    '''Returns a random color (4 tuple).

    :Parameters:
        `alpha`: float, defaults to 1.0
            If alpha == 'random', a random alpha value is generated.
    '''
    return [random() * 255,
            random() * 255, 
            random() * 255, 
            random() * 255 if alpha == 'random' else alpha]


def is_color_transparent(*rgb: list[int]):
    '''Return True if the alpha channel is 0.'''
    if len(rgb) < 4:
        return False

    if float(rgb[3]) == 0.:
        return True

    return False


def print_hex_color(*rgb):
    '''
    arg: list[int]
        - (int, int, int, int)
        - (int, int, int)

    Return colored text.
    '''
    print('\033[38;2;{};{};{}m{}\033[0m'.format(rgb[0], rgb[1], rgb[2], rgb))


#!/usr/bin/python3

import os
from PySide6.QtCore import QThread, Signal, QUrl
from PySide6.QtWidgets import QProgressBar, QWidget
from ..core import BehaviorDeclarative, QObjectBase


class CopyProgress(BehaviorDeclarative):
    def __init__(self,
                 *,
                 child: QProgressBar | QWidget,
                 src: str | QUrl,
                 dst: str | QUrl,
                 filename: str = None,
                 running: bool = False,
                 chunk_size: int = 1024 * (1024 // 2),
                 overwrite: bool = False,
                 **kwargs) -> QWidget:

        for method in (
            "maximum",
            "setValue"
        ):
            if not hasattr(child, method):
                raise ValueError("invalid progress bar widget!")

        self.child = child
        self.copy_thread = CopyThread(self.child, 
                                      src, 
                                      dst,
                                      filename, 
                                      chunk_size,
                                      overwrite,
                                      **kwargs)

        self.copy_thread.copy_progress.connect(self.update_progress)

        if running:
            self.copy_thread.start()

        return self.child
    
    def set_src(self, path: str | QUrl):
        self.copy_thread.set_src(path)
    
    def set_dst(self, path: str | QUrl):
        self.copy_thread.set_dst(path)

    def start(self):
        self.copy_thread.start()

    def pause(self):
        self.copy_thread.pause()

    def resum(self):
        self.copy_thread.resum()

    def update_progress(self, progress: int):
        self.child.setValue(progress)

    def build(self):
        return self.copy_thread


class CopyThread(QObjectBase, QThread):
    copy_done = Signal()
    copy_paused = Signal()
    copy_progress = Signal(int)

    def __init__(self, 
                 parent, 
                 src, 
                 dst, 
                 filename, 
                 chunk_size,
                 overwrite,
                 **kwargs):

        QThread.__init__(self, parent)
        super().__init__(**kwargs)

        self.set_src(src)
        self.set_dst(dst)

        self.filename = filename
        self.chunk_size = chunk_size
        self.overwrite = overwrite
        self._is_paused: bool = False

    def set_src(self, path: str | QUrl):
        self.src = (path.toLocalFile() 
                    if isinstance(path, QUrl) 
                    else path)

    def set_dst(self, path: str | QUrl):
        self.dst = (path.toLocalFile() 
                    if isinstance(path, QUrl) 
                    else path)

    def get_filename(self):
        if self.filename is not None:
            return self.filename

        return os.path.basename(self.src)

    @property
    def file_destination(self):
        return os.path.join(self.dst, self.get_filename())

    def run(self):
        self.copy()

    def pause(self):
        self._is_paused = True
        self.copy_paused.emit()

    def resum(self):
        self._is_paused = False

        if not os.path.exists(self.src):
            raise FileNotFoundError(f"'{self.src}' does not exist!")

        if not os.path.exists(self.file_destination):
            raise FileNotFoundError(f"'{self.file_destination}' does not exist!")

        source_size = os.path.getsize(self.src)
        dest_size = os.path.getsize(self.file_destination)
        copied = dest_size

        with (open(self.src, "rb") as source, 
              open(self.file_destination, "ab") as target):

            source.seek(dest_size, 1)

            while True:
                chunk = source.read(self.chunk_size)
                if not chunk:
                    break

                target.write(chunk)
                copied += len(chunk)
                progress = copied * (self.parent().maximum() 
                                        if self.parent() is not None 
                                        else 100) / source_size

                self.copy_progress.emit(progress)

                if progress >= 100:
                    self.copy_done.emit()

    def copy(self):
        if os.path.exists(self.file_destination) and not self.overwrite:
            raise FileExistsError(self.file_destination)

        source_size = os.path.getsize(self.src)
        copied = 0

        with (open(self.src, "rb") as source,
              open(self.file_destination, "wb") as target):

            while True:
                if self._is_paused:
                    break

                chunk = source.read(self.chunk_size)
                if not chunk:
                    self.copy_done.emit()
                    break

                target.write(chunk)
                copied += len(chunk)
                progress = copied * (self.parent().maximum() 
                                     if self.parent() is not None 
                                     else 100) / source_size

                self.copy_progress.emit(progress)

                # if progress >= 100:
                #     self.copy_done.emit()


from PySide6.QtWidgets import QStackedLayout, QWidget, QLayoutItem
from ..core import QObjectBase
from typing import Union


class StackedLayout(QObjectBase, QStackedLayout):
    def __init__(self,
                 *,
                 children: list[Union[QWidget, QLayoutItem]] = None,
                 **kwargs):
        QStackedLayout.__init__(self)
        super().__init__(**kwargs)

        self._set_children(children)

    def _set_children(self, children: list[Union[QWidget, QLayoutItem]]) -> None:
        if not children:
            return

        for child in children:
            if isinstance(child, QLayoutItem):
                self.addItem(child)

            elif isinstance(child, QWidget):
                self.addWidget(child)


# coding: utf-8
from enum import EnumMeta
from functools import singledispatch, update_wrapper


class singledispatchmethod:
    """Single-dispatch generic method descriptor.

    Supports wrapping existing descriptors and handles non-descriptor
    callables as instance methods.
    
    e.g:

    @singledispatchmethod
    def __init__(self):
        super().__init__(parent, **kwargs)

    @__init__.register
    def _(self, *args):
        super().__init__(text, parent, **kwargs)
        self.fluentIcon = None

    @__init__.register
    def _(self, *args, **kwargs):
        super().__init__(icon, text, parent, **kwargs)
        self.fluentIcon = None
    """

    def __init__(self, func):
        if not callable(func) and not hasattr(func, "__get__"):
            raise TypeError(f"{func!r} is not callable or a descriptor")

        self.dispatcher = singledispatch(func)
        self.func = func

    def register(self, cls, method=None):
        """generic_method.register(cls, func) -> func

        Registers a new implementation for the given *cls* on a *generic_method*.
        """
        return self.dispatcher.register(cls, func=method)

    def __get__(self, obj, cls=None):
        def _method(*args, **kwargs):
            if args:
                method = self.dispatcher.dispatch(args[0].__class__)
            else:
                method = self.func
                for v in kwargs.values():
                    if v.__class__ in self.dispatcher.registry:
                        method = self.dispatcher.dispatch(v.__class__)
                        if method is not self.func:
                            break

            return method.__get__(obj, cls)(*args, **kwargs)

        _method.__isabstractmethod__ = self.__isabstractmethod__
        _method.register = self.register
        update_wrapper(_method, self.func)
        return _method

    @property
    def __isabstractmethod__(self):
        return getattr(self.func, '__isabstractmethod__', False)


class staticproperty(property):
    '''
    class Object:
        @staticproperty
        def some_method():
            ...
    '''

    def __get__(self, owner_self, owner_cls):         
        return self.fget()


class classproperty(property):
    '''
    class Object:
        
        @classproperty
        def some_method(cls):
            ...
    '''

    def __get__(self, owner_self, owner_cls):
        return self.fget(owner_cls)


class QueryDict(dict):
    '''QueryDict is a dict() that can be queried with dot.

    ::

        d = QueryDict()
        
        # create a key named toto, with the value 1
        d.toto = 1

        # it's the same as
        d['toto'] = 1

    .. versionadded:: 1.0.4
    '''

    def __getattr__(self, attr):
        try:
            return self.__getitem__(attr)
        except KeyError:
            return super(QueryDict, self).__getattr__(attr)

    def __setattr__(self, attr, value):
        self.__setitem__(attr, value)


class EnumDirectValueMeta(EnumMeta):
    '''
    class EnumClass(Enum, metaclass=EnumDirectValueMeta):
        red = 0
        green = 1
        blue = 2

    >> EnumClass.red
    0
    '''
    def __getattribute__(cls, name):
        value = super().__getattribute__(name)
        if isinstance(value, cls):
            value = value.value
        return value


from PySide6.QtWidgets import QGraphicsColorizeEffect, QWidget
from ..core import QObjectBase


class ColorizeEffect(QObjectBase, QGraphicsColorizeEffect):
    def __init__(self, **kwargs) -> QWidget:
        QGraphicsColorizeEffect.__init__(self)
        super().__init__(**kwargs)


from PySide6.QtCore import QByteArray, QSize, Qt
from PySide6.QtGui import QColor, QMovie
from PySide6.QtWidgets import QLabel
from ..core import WidgetBase
import os


class MovieView(WidgetBase, QLabel):
    def __init__(self,
                 *,
                 filename: str = None,
                 bg_color: QColor = None,
                 scale_size: QSize = None,
                 speed: int = None,
                 running: bool = False,
                 **kwargs):
        QLabel.__init__(self)
        super().__init__(**kwargs)

        if (QByteArray(bytes(
            os.path.basename(filename).split(".")[-1].lower(), "utf-8")) 
            not in QMovie.supportedFormats()):
            raise ValueError("invalid filename, unsupported format!")

        self.setMouseTracking(True)
        self.setAlignment(Qt.AlignmentFlag.AlignCenter)

        self._movie = QMovie(self)

        if filename is not None:
            self._movie.setFileName(filename)

        if bg_color is not None:
            self._movie.setBackgroundColor(bg_color)

        if scale_size is not None:
            self._movie.setScaledSize(scale_size)

        if speed is not None:
            self._movie.setSpeed(speed)

        if running:
            self._movie.start()

        self.setMovie(self._movie)



#!/usr/bin/python3

from PySide6.QtWidgets import QGridLayout, QWidget, QApplication, QLabel, QDialog, QWidget
from PySide6.QtGui import QFont, QIcon, QPixmap
from PySide6.QtCore import QSize, QTimer, Qt
from ..eliding_label import ElidingLabel


class _SilentTextDialog(QDialog):
    def __init__(self, 
                 parent = None, 
                 close_on_mouse_press: bool = True) -> None:
        QDialog.__init__(self, parent)

        self._close_on_mouse_press = close_on_mouse_press

        self.setWindowFlags(self.windowFlags() | Qt.WindowType.FramelessWindowHint)
        self.setAttribute(Qt.WidgetAttribute.WA_TranslucentBackground, True)
        self.setAttribute(Qt.WidgetAttribute.WA_AlwaysStackOnTop, True)
        self.setWindowOpacity(0.9)

        self.setStyleSheet("""
            QDialog, QWidget {
                padding: 6px;
                border-radius: 12px;
                background-color: #0d0d0d;
                color: #ffffff;
            }
        """)

        ## Create Qt Widgetes
        self.form = QWidget(self)
        self.form.setAttribute(Qt.WidgetAttribute.WA_TranslucentBackground, True)

        self.grid_layout1 = QGridLayout(self)
        self.grid_layout2 = QGridLayout(self.form)

        self.icon = QLabel(self.form)
        self.text = ElidingLabel(elide_mode=Qt.TextElideMode.ElideRight)

        font = QFont()
        font.setBold(True)
        font.setPixelSize(16)

        self.text.setFont(font)
        self.text.setAlignment(Qt.AlignmentFlag.AlignCenter)

        self.grid_layout2.addWidget(self.icon, 0, 0, 1, 1, Qt.AlignmentFlag.AlignLeft)
        self.grid_layout2.addWidget(self.text, 0, 1, 1, 1, Qt.AlignmentFlag.AlignCenter)

        self.form.setLayout(self.grid_layout2)
        self.grid_layout1.addWidget(self.form)
        self.setLayout(self.grid_layout1)

    def start_closeing(self):
        self.close()

    def start_show(self):
        if self.isHidden():
            self.show()
            self.adjustSize()

    def display_text(self, 
                     icon: QIcon | QPixmap,
                     text: str,
                     timeout: int = 1500, 
                     icon_size: QSize = None):

        self.text.setText(text)
        if isinstance(icon, QIcon):
            self.icon.setPixmap(icon.pixmap(icon_size 
                                            if icon_size is not None 
                                            else QSize(32, 32)))
        else:
            self.icon.setPixmap(icon)

        QTimer.singleShot(0, self.start_show)
        QTimer.singleShot(timeout, self.start_closeing)

    def mousePressEvent(self, event) -> None:
        if self._close_on_mouse_press:
            self.start_closeing()
        return super().mousePressEvent()


class SilentTextDialog:
    @classmethod
    def display(cls,
                *,
                icon: QIcon | QPixmap,
                text: str,
                timeout: int = 1500,
                icon_size: QSize = None,
                auto_close: bool = False,
                parent: QWidget = None,
                **kwargs):

        if not parent:
            parent = QApplication.activeWindow()

        silent_dialog = _SilentTextDialog(parent, auto_close, **kwargs)
        silent_dialog.display_text(icon, text, timeout, icon_size)